import type { CaptureType } from '@microsoft/fast-element';
import { Constructable } from '@microsoft/fast-element';
import { CSSDirective } from '@microsoft/fast-element';
import { Direction } from '@microsoft/fast-web-utilities';
import type { ElementsFilter } from '@microsoft/fast-element';
import { ElementStyles } from '@microsoft/fast-element';
import { ElementViewTemplate } from '@microsoft/fast-element';
import { FASTElement } from '@microsoft/fast-element';
import { FASTElementDefinition } from '@microsoft/fast-element';
import { HostBehavior } from '@microsoft/fast-element';
import { HostController } from '@microsoft/fast-element';
import { HTMLDirective } from '@microsoft/fast-element';
import { Orientation } from '@microsoft/fast-web-utilities';
import { SyntheticViewTemplate } from '@microsoft/fast-element';
import { ViewTemplate } from '@microsoft/fast-element';

/**
 * Expand mode for {@link FASTAccordion}
 * @public
 */
export declare const AccordionExpandMode: {
    /**
     * Designates only a single {@link @microsoft/fast-foundation#(FASTAccordionItem:class) } can be open a time.
     */
    readonly single: "single";
    /**
     * Designates multiple {@link @microsoft/fast-foundation#(FASTAccordionItem:class) | FASTAccordionItemItems} can be open simultaneously.
     */
    readonly multi: "multi";
};

/**
 * Type for the {@link FASTAccordion} Expand Mode
 * @public
 */
export declare type AccordionExpandMode = typeof AccordionExpandMode[keyof typeof AccordionExpandMode];

/**
 * Accordion Item configuration options
 * @public
 */
export declare type AccordionItemOptions = StartEndOptions & {
    expandedIcon?: string | SyntheticViewTemplate;
    collapsedIcon?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTAccordionItem:class)} component.
 * @public
 */
export declare function accordionItemTemplate<T extends FASTAccordionItem>(options?: AccordionItemOptions): ElementViewTemplate<T>;

/**
 * Creates a template for the {@link @microsoft/fast-foundation#FASTAccordion} component.
 * @public
 */
export declare function accordionTemplate<T extends FASTAccordion>(): ElementViewTemplate<T>;

/**
 * A utility interface to store anchored region
 * configurations that correspond to various common flyout
 * positioning schemes
 *
 * @public
 */
export declare interface AnchoredRegionConfig {
    /**
     * Whether the region is positioned using css "position: fixed".
     * Otherwise the region uses "position: absolute".
     * Fixed placement allows the region to break out of parent containers,
     */
    readonly fixedPlacement?: boolean;
    /**
     * The auto-update setting of the component
     */
    readonly autoUpdateMode?: AutoUpdateMode;
    /**
     * Sets what logic the component uses to determine vertical placement.
     */
    readonly verticalPositioningMode?: AxisPositioningMode;
    /**
     * The default vertical position of the region relative to the anchor element
     */
    readonly verticalDefaultPosition?: VerticalPosition;
    /**
     * Whether the region overlaps the anchor on the vertical axis
     */
    readonly verticalInset?: boolean;
    /**
     * Defines how the height of the region is calculated
     */
    readonly verticalScaling?: AxisScalingMode;
    /**
     * How short the space allocated to the default position has to be before the tallest area
     * is selected for layout
     */
    readonly verticalThreshold?: number;
    /**
     * Whether the region remains in the viewport (ie. detaches from the anchor) on the vertical axis
     */
    readonly verticalViewportLock?: boolean;
    /**
     * Sets what logic the component uses to determine horizontal placement.
     */
    readonly horizontalPositioningMode?: AxisPositioningMode;
    /**
     * The default horizontal position of the region relative to the anchor element
     */
    readonly horizontalDefaultPosition?: HorizontalPosition;
    /**
     *  hether the region overlaps the anchor on the horizontal axis
     */
    readonly horizontalInset?: boolean;
    /**
     * Defines how the width of the region is calculate
     */
    readonly horizontalScaling?: AxisScalingMode;
    /**
     * Whether the region remains in the viewport (ie. detaches from the anchor) on the horizontal axis
     */
    readonly horizontalViewportLock?: boolean;
    /**
     * How short the space allocated to the default position has to be before the widest area
     * is selected for layout
     */
    readonly horizontalThreshold?: number;
}

/**
 * Values to describe the possible positions of the region relative to its anchor.
 * Depending on the axis start = left/top, end = right/bottom
 * @public
 */
export declare const AnchoredRegionPositionLabel: {
    readonly start: "start";
    readonly insetStart: "insetStart";
    readonly insetEnd: "insetEnd";
    readonly end: "end";
    readonly center: "center";
};

/**
 * Describes the possible positions of the region relative
 * to its anchor. Depending on the axis start = left/top, end = right/bottom
 *
 * @public
 */
export declare type AnchoredRegionPositionLabel = typeof AnchoredRegionPositionLabel[keyof typeof AnchoredRegionPositionLabel];

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTAnchoredRegion:class)} component.
 * @public
 */
export declare function anchoredRegionTemplate<T extends FASTAnchoredRegion>(): ElementViewTemplate<T>;

/**
 * Anchor configuration options
 * @public
 */
export declare type AnchorOptions = StartEndOptions;

/**
 * Anchor target values.
 *
 * @public
 */
export declare const AnchorTarget: {
    readonly _self: "_self";
    readonly _blank: "_blank";
    readonly _parent: "_parent";
    readonly _top: "_top";
};

/**
 * Type for anchor target values.
 *
 * @public
 */
export declare type AnchorTarget = typeof AnchorTarget[keyof typeof AnchorTarget];

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTAnchor:class)} component.
 * @public
 */
export declare function anchorTemplate<T extends FASTAnchor>(options?: AnchorOptions): ElementViewTemplate<T>;

/**
 * Apply mixins to a constructor.
 * Sourced from {@link https://www.typescriptlang.org/docs/handbook/mixins.html | TypeScript Documentation }.
 * @public
 */
export declare function applyMixins(derivedCtor: any, ...baseCtors: any[]): void;

/**
 * Some states and properties are applicable to all host language elements regardless of whether a role is applied.
 * The following global states and properties are supported by all roles and by all base markup elements.
 * {@link https://www.w3.org/TR/wai-aria-1.1/#global_states}
 *
 * This is intended to be used as a mixin. Be sure you extend FASTElement.
 *
 * @public
 */
export declare class ARIAGlobalStatesAndProperties {
    /**
     * Indicates whether assistive technologies will present all, or only parts of,
     * the changed region based on the change notifications defined by the aria-relevant attribute.
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-atomic}
     *
     * @public
     * @remarks
     * HTML Attribute: aria-atomic
     */
    ariaAtomic: "true" | "false" | string | null;
    /**
     * Indicates an element is being modified and that assistive technologies MAY want to wait
     * until the modifications are complete before exposing them to the user.
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-busy}
     *
     * @public
     * @remarks
     * HTML Attribute: aria-busy
     */
    ariaBusy: "true" | "false" | string | null;
    /**
     * Identifies the element (or elements) whose contents or presence are controlled by the current element.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-controls}
     * @public
     * @remarks
     * HTML Attribute: aria-controls
     */
    ariaControls: string | null;
    /**
     * Indicates the element that represents the current item within a container or set of related elements.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-current}
     * @public
     * @remarks
     * HTML Attribute: aria-current
     */
    ariaCurrent: "page" | "step" | "location" | "date" | "time" | "true" | "false" | string | null;
    /**
     * Identifies the element (or elements) that describes the object.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-describedby}
     * @public
     * @remarks
     * HTML Attribute: aria-describedby
     */
    ariaDescribedby: string | null;
    /**
     * Identifies the element that provides a detailed, extended description for the object.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-details}
     * @public
     * @remarks
     * HTML Attribute: aria-details
     */
    ariaDetails: string | null;
    /**
     * Indicates that the element is perceivable but disabled, so it is not editable or otherwise operable.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-disabled}
     * @public
     * @remarks
     * HTML Attribute: aria-disabled
     */
    ariaDisabled: "true" | "false" | string | null;
    /**
     * Identifies the element that provides an error message for the object.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-errormessage}
     * @public
     * @remarks
     * HTML Attribute: aria-errormessage
     */
    ariaErrormessage: string | null;
    /**
     * Identifies the next element (or elements) in an alternate reading order of content which, at the user's discretion,
     * allows assistive technology to override the general default of reading in document source order.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-flowto}
     * @public
     * @remarks
     * HTML Attribute: aria-flowto
     */
    ariaFlowto: string | null;
    /**
     * Indicates the availability and type of interactive popup element,
     * such as menu or dialog, that can be triggered by an element.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-haspopup}
     * @public
     * @remarks
     * HTML Attribute: aria-haspopup
     */
    ariaHaspopup: "false" | "true" | "menu" | "listbox" | "tree" | "grid" | "dialog" | string | null;
    /**
     * Indicates whether the element is exposed to an accessibility API
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-hidden}
     * @public
     * @remarks
     * HTML Attribute: aria-hidden
     */
    ariaHidden: "false" | "true" | string | null;
    /**
     * Indicates the entered value does not conform to the format expected by the application.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-invalid}
     * @public
     * @remarks
     * HTML Attribute: aria-invalid
     */
    ariaInvalid: "false" | "true" | "grammar" | "spelling" | string | null;
    /**
     * Indicates keyboard shortcuts that an author has implemented to activate or give focus to an element.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-keyshortcuts}
     * @public
     * @remarks
     * HTML Attribute: aria-keyshortcuts
     */
    ariaKeyshortcuts: string | null;
    /**
     * Defines a string value that labels the current element.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-label}
     * @public
     * @remarks
     * HTML Attribute: aria-label
     */
    ariaLabel: string | null;
    /**
     * Identifies the element (or elements) that labels the current element.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-labelledby}
     * @public
     * @remarks
     * HTML Attribute: aria-labelledby
     */
    ariaLabelledby: string | null;
    /**
     * Indicates that an element will be updated, and describes the types of updates the user agents,
     * assistive technologies, and user can expect from the live region.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-live}
     * @public
     * @remarks
     * HTML Attribute: aria-live
     */
    ariaLive: "assertive" | "off" | "polite" | string | null;
    /**
     * Identifies an element (or elements) in order to define a visual,
     * functional, or contextual parent/child relationship between DOM elements
     * where the DOM hierarchy cannot be used to represent the relationship.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-owns}
     * @public
     * @remarks
     * HTML Attribute: aria-owns
     */
    ariaOwns: string | null;
    /**
     * Indicates what notifications the user agent will trigger when the accessibility tree within a live region is modified.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-relevant}
     * @public
     * @remarks
     * HTML Attribute: aria-relevant
     */
    ariaRelevant: "additions" | "additions text" | "all" | "removals" | "text" | string | null;
    /**
     * Defines a human-readable, author-localized description for the role of an element.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-roledescription}
     * @public
     * @remarks
     * HTML Attribute: aria-roledescription
     */
    ariaRoledescription: string | null;
}

/**
 * Defines if the component updates its position automatically. Calling update() always provokes an update.
 * anchor - the component only updates its position when the anchor resizes (default)
 * auto - the component updates its position when:
 * - update() is called
 * - the anchor resizes
 * - the window resizes
 * - the viewport resizes
 * - any scroll event in the document
 *
 * @public
 */
export declare const AutoUpdateMode: {
    readonly anchor: "anchor";
    readonly auto: "auto";
};

/**
 * Type for the auto update mode values
 * @public
 */
export declare type AutoUpdateMode = typeof AutoUpdateMode[keyof typeof AutoUpdateMode];

/**
 * Avatar configuration options
 * @public
 */
export declare type AvatarOptions = {
    media?: string | SyntheticViewTemplate;
};

/**
 * The template for {@link @microsoft/fast-foundation#FASTAvatar} component.
 * @public
 */
export declare function avatarTemplate<T extends FASTAvatar>(options?: AvatarOptions): ElementViewTemplate<T>;

/**
 * Values to define the base behavior of an anchored region on a particular axis
 * @public
 */
export declare const AxisPositioningMode: {
    readonly uncontrolled: "uncontrolled";
    readonly locktodefault: "locktodefault";
    readonly dynamic: "dynamic";
};

/**
 * Type to define the base behavior of an anchored region on a particular axis
 * @public
 */
export declare type AxisPositioningMode = typeof AxisPositioningMode[keyof typeof AxisPositioningMode];

/**
 * Values to define the scaling behavior of an anchored region on a particular axis
 * @public
 */
export declare const AxisScalingMode: {
    readonly anchor: "anchor";
    readonly content: "content";
    readonly fill: "fill";
};

/**
 * Type to define the scaling behavior of an anchored region on a particular axis
 *
 * @public
 */
export declare type AxisScalingMode = typeof AxisScalingMode[keyof typeof AxisScalingMode];

/**
 * The template for the {@link @microsoft/fast-foundation#FASTBadge} component.
 * @public
 */
export declare function badgeTemplate<T extends FASTBadge>(): ElementViewTemplate<T>;

/**
 * Breadcrumb Item configuration options
 *
 * @slot - The default slot for when no href is provided or for providing your own custom elements
 * @slot separator - The slot for providing a custom separator
 * @csspart listitem - The wrapping container for the item, represents a semantic listitem
 * @csspart separator - The wrapping element for the separator
 *
 * @public
 */
export declare type BreadcrumbItemOptions = StartEndOptions & {
    separator?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTBreadcrumbItem:class)} component.
 * @public
 */
export declare function breadcrumbItemTemplate<T extends FASTBreadcrumbItem>(options?: BreadcrumbItemOptions): ElementViewTemplate<T>;

/**
 * The template for the {@link @microsoft/fast-foundation#FASTBreadcrumb} component.
 * @public
 */
export declare function breadcrumbTemplate<T extends FASTBreadcrumb>(): ElementViewTemplate<T>;

declare class _Button extends FASTElement {
}

declare interface _Button extends FormAssociated {
}

/**
 * Button configuration options
 * @public
 */
export declare type ButtonOptions = StartEndOptions;

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTButton:class)} component.
 * @public
 */
export declare function buttonTemplate<T extends FASTButton>(options?: ButtonOptions): ElementViewTemplate<T>;

/**
 * Button type values.
 *
 * @public
 */
export declare const ButtonType: {
    readonly submit: "submit";
    readonly reset: "reset";
    readonly button: "button";
};

/**
 * Type for button type values.
 *
 * @public
 */
export declare type ButtonType = typeof ButtonType[keyof typeof ButtonType];

/**
 * A calendar day template
 * @param context - Element definition context for getting the cell tag for calendar-cell
 * @param todayString - A string representation for todays date
 * @returns - A calendar cell template for a given date
 * @public
 */
export declare function calendarCellTemplate(options: CalendarOptions, todayString: string): ViewTemplate<CalendarDateInfo>;

/**
 * Caldendar date info
 * used to represent a date
 * @public
 */
export declare type CalendarDateInfo = {
    day: number;
    month: number;
    year: number;
    disabled?: boolean;
    selected?: boolean;
};

/**
 * Calendar information needed for rendering
 * including the next and previous months
 * @public
 */
export declare type CalendarInfo = MonthInfo & {
    previous: MonthInfo;
    next: MonthInfo;
};

/**
 * Calendar configuration options
 * @public
 */
export declare type CalendarOptions = StartEndOptions & {
    dataGridCell: TemplateElementDependency;
    dataGridRow: TemplateElementDependency;
    dataGrid: TemplateElementDependency;
    title?: SyntheticViewTemplate | string;
};

/**
 *
 * @param context - Element definition context for getting the cell tag for calendar-cell
 * @param todayString - A string representation for todays date
 * @returns - A template for a week of days
 * @public
 */
export declare function calendarRowTemplate(options: CalendarOptions, todayString: string): ViewTemplate;

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTCalendar:class)} component.
 *
 * @param context - Element definition context for getting the cell tag for calendar-cell
 * @param definition - Foundation element definition
 * @returns - a template for a calendar month
 * @public
 */
export declare function calendarTemplate<T extends FASTCalendar>(options: CalendarOptions): ElementViewTemplate<T>;

/**
 * A basic Calendar title template that includes the month and year
 * @returns - A calendar title template
 * @public
 */
export declare function calendarTitleTemplate<T extends FASTCalendar>(): ViewTemplate<T>;

/**
 * Calendar weekday label template
 * @returns - The weekday labels template
 * @public
 */
export declare function calendarWeekdayTemplate(options: CalendarOptions): ViewTemplate<WeekdayText>;

/**
 * The template for the {@link @microsoft/fast-foundation#FASTCard} component.
 * @public
 */
export declare function cardTemplate<T extends FASTCard>(): ElementViewTemplate<T>;

/**
 * Options for data grid cells.
 * @public
 */
export declare type CellItemTemplateOptions = {
    dataGridCell: TemplateElementDependency;
};

/**
 * Creates a checkable form associated component.
 * @beta
 */
export declare function CheckableFormAssociated<T extends ConstructableFormAssociated>(BaseCtor: T): T;

/**
 * Base class for providing Custom Element Form Association with checkable features.
 *
 * @beta
 */
export declare interface CheckableFormAssociated extends FormAssociated {
    currentChecked: boolean;
    dirtyChecked: boolean;
    checkedAttribute: boolean;
    defaultChecked: boolean;
    defaultCheckedChanged(oldValue: boolean | undefined, newValue: boolean): void;
    checked: boolean;
    checkedChanged(oldValue: boolean | undefined, newValue: boolean): void;
}

/**
 * Combined type to describe a checkable Form-associated element.
 *
 * @beta
 */
export declare type CheckableFormAssociatedElement = FormAssociatedElement & CheckableFormAssociated & {
    proxy: HTMLInputElement;
};

declare class _Checkbox extends FASTElement {
}

declare interface _Checkbox extends CheckableFormAssociated {
}

/**
 * Checkbox configuration options
 * @public
 */
export declare type CheckboxOptions = {
    checkedIndicator?: string | SyntheticViewTemplate;
    indeterminateIndicator?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTCheckbox:class)} component.
 * @public
 */
export declare function checkboxTemplate<T extends FASTCheckbox>(options?: CheckboxOptions): ElementViewTemplate<T>;

/**
 * Defines a column in the grid
 *
 * @public
 */
export declare interface ColumnDefinition {
    /**
     * Identifies the data item to be displayed in this column
     * (i.e. how the data item is labelled in each row)
     */
    columnDataKey: string;
    /**
     * Sets the css grid-column property on the cell which controls its placement in
     * the parent row. If left unset the cells will set this value to match the index
     * of their column in the parent collection of ColumnDefinitions.
     */
    gridColumn?: string;
    /**
     *  Column title, if not provided columnDataKey is used as title
     */
    title?: string;
    /**
     *  Header cell template
     */
    headerCellTemplate?: ViewTemplate | SyntheticViewTemplate;
    /**
     * Whether the header cell has an internal focus queue
     */
    headerCellInternalFocusQueue?: boolean;
    /**
     * Callback function that returns the element to focus in a custom cell.
     * When headerCellInternalFocusQueue is false this function is called when the cell is first focused
     * to immediately move focus to a cell element, for example a cell that is a checkbox could move
     * focus directly to the checkbox.
     * When headerCellInternalFocusQueue is true this function is called when the user hits Enter or F2
     */
    headerCellFocusTargetCallback?: (cell: FASTDataGridCell) => HTMLElement;
    /**
     * cell template
     */
    cellTemplate?: ViewTemplate | SyntheticViewTemplate;
    /**
     * Whether the cell has an internal focus queue
     */
    cellInternalFocusQueue?: boolean;
    /**
     * Callback function that returns the element to focus in a custom cell.
     * When cellInternalFocusQueue is false this function is called when the cell is first focused
     * to immediately move focus to a cell element, for example a cell that is a checkbox could move
     * focus directly to the checkbox.
     * When cellInternalFocusQueue is true this function is called when the user hits Enter or F2
     */
    cellFocusTargetCallback?: (cell: FASTDataGridCell) => HTMLElement;
    /**
     * Whether this column is the row header
     */
    isRowHeader?: boolean;
}

declare class _Combobox extends FASTListbox {
}

declare interface _Combobox extends FormAssociated {
}

/**
 * Autocomplete values for combobox.
 * @public
 */
export declare const ComboboxAutocomplete: {
    readonly inline: "inline";
    readonly list: "list";
    readonly both: "both";
    readonly none: "none";
};

/**
 * Autocomplete type for combobox.
 * @public
 */
export declare type ComboboxAutocomplete = typeof ComboboxAutocomplete[keyof typeof ComboboxAutocomplete];

/**
 * Combobox configuration options
 * @public
 */
export declare type ComboboxOptions = StartEndOptions & {
    indicator?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTCombobox:class)} component.
 * @public
 */
export declare function comboboxTemplate<T extends FASTCombobox>(options?: ComboboxOptions): ElementViewTemplate<T>;

/**
 * Combined type to describe a Constructable Form-Associated type.
 *
 * @beta
 */
export declare type ConstructableFormAssociated = Constructable<HTMLElement & FASTElement>;

/**
 * @public
 */
export declare class CSSDesignToken<T> extends DesignToken<T> implements CSSDirective, HTMLDirective {
    /**
     * The CSS Custom property name of the token.
     */
    readonly cssCustomProperty: string;
    private cssVar;
    /**
     * The DesignToken represented as a string that can be used in CSS.
     */
    createCSS(): string;
    /**
     * Creates HTML to be used within a template.
     */
    createHTML(): string;
    private cssReflector;
    constructor(configuration: CSSDesignTokenConfiguration);
    private resolveCSSValue;
}

/**
 * @public
 */
export declare interface CSSDesignTokenConfiguration extends DesignTokenConfiguration {
    /**
     * The name of the CSS custom property to associate to the {@link CSSDesignToken}
     */
    cssCustomPropertyName: string;
}

/**
 * Define all possible CSS display values.
 * @public
 * @deprecated - fast-foundation is removing styling utilities
 */
export declare type CSSDisplayPropertyValue = "block" | "contents" | "flex" | "grid" | "inherit" | "initial" | "inline" | "inline-block" | "inline-flex" | "inline-grid" | "inline-table" | "list-item" | "none" | "run-in" | "table" | "table-caption" | "table-cell" | "table-column" | "table-column-group" | "table-footer-group" | "table-header-group" | "table-row" | "table-row-group";

/**
 * This can be used to construct a behavior to apply a prefers color scheme: dark only stylesheet.
 * @public
 */
export declare const darkModeStylesheetBehavior: (styles: ElementStyles) => MatchMediaStyleSheetBehavior;

/**
 * Generates a template for the {@link @microsoft/fast-foundation#FASTDataGridCell} component using
 * the provided prefix.
 * @public
 */
export declare function dataGridCellTemplate<T extends FASTDataGridCell>(): ElementViewTemplate<T>;

/**
 * Enumerates possible data grid cell types.
 *
 * @public
 */
export declare const DataGridCellTypes: {
    readonly default: "default";
    readonly columnHeader: "columnheader";
    readonly rowHeader: "rowheader";
};

/**
 * The possible cell types.
 *
 * @public
 */
export declare type DataGridCellTypes = typeof DataGridCellTypes[keyof typeof DataGridCellTypes];

/**
 * Options for data grid templates.
 * @public
 */
export declare type DataGridOptions = {
    dataGridRow: TemplateElementDependency;
};

/**
 * Generates a template for the {@link @microsoft/fast-foundation#FASTDataGridRow} component using
 * the provided prefix.
 *
 * @public
 */
export declare function dataGridRowTemplate<T extends FASTDataGridRow>(options: CellItemTemplateOptions): ElementViewTemplate<T>;

/**
 * Enumerates possible data grid row types
 *
 * @public
 */
export declare const DataGridRowTypes: {
    readonly default: "default";
    readonly header: "header";
    readonly stickyHeader: "sticky-header";
};

/**
 * The possible data grid row types
 *
 * @public
 */
export declare type DataGridRowTypes = typeof DataGridRowTypes[keyof typeof DataGridRowTypes];

/**
 * Generates a template for the {@link @microsoft/fast-foundation#FASTDataGrid} component using
 * the provided prefix.
 *
 * @public
 */
export declare function dataGridTemplate<T extends FASTDataGrid>(options: DataGridOptions): ElementViewTemplate<T>;

/**
 * Date formatting utility
 * @public
 */
export declare class DateFormatter {
    /**
     * Localization settings to use for formatting
     * @public
     */
    locale: string;
    /**
     * Formatting for the day
     * @public
     */
    dayFormat: DayFormat;
    /**
     * Formatting for the weekday labels
     * @public
     */
    weekdayFormat: WeekdayFormat;
    /**
     * Formatting for the month
     * @public
     */
    monthFormat: MonthFormat;
    /**
     * Formatting for the year
     * @public
     */
    yearFormat: YearFormat;
    /**
     * Date used for formatting
     */
    date: Date;
    constructor(config?: {});
    /**
     * Helper function to make sure that the DateFormatter is working with an instance of Date
     * @param date - The date as an object, string or Date insance
     * @returns - A Date instance
     * @public
     */
    getDateObject(date: {
        day: number;
        month: number;
        year: number;
    } | string | Date): Date;
    /**
     *
     * @param date - a valide date as either a Date, string, objec or a DateFormatter
     * @param format - The formatting for the string
     * @param locale - locale data used for formatting
     * @returns A localized string of the date provided
     * @public
     */
    getDate(date?: {
        day: number;
        month: number;
        year: number;
    } | string | Date, format?: Intl.DateTimeFormatOptions, locale?: string): string;
    /**
     *
     * @param day - Day to localize
     * @param format - The formatting for the day
     * @param locale - The locale data used for formatting
     * @returns - A localized number for the day
     * @public
     */
    getDay(day?: number, format?: DayFormat, locale?: string): string;
    /**
     *
     * @param month - The month to localize
     * @param format - The formatting for the month
     * @param locale - The locale data used for formatting
     * @returns - A localized name of the month
     * @public
     */
    getMonth(month?: number, format?: MonthFormat, locale?: string): string;
    /**
     *
     * @param year - The year to localize
     * @param format - The formatting for the year
     * @param locale - The locale data used for formatting
     * @returns - A localized string for the year
     * @public
     */
    getYear(year?: number, format?: YearFormat, locale?: string): string;
    /**
     *
     * @param weekday - The number of the weekday, defaults to Sunday
     * @param format - The formatting for the weekday label
     * @param locale - The locale data used for formatting
     * @returns - A formatted weekday label
     * @public
     */
    getWeekday(weekday?: number, format?: WeekdayFormat, locale?: string): string;
    /**
     *
     * @param format - The formatting for the weekdays
     * @param locale - The locale data used for formatting
     * @returns - An array of the weekday labels
     * @public
     */
    getWeekdays(format?: WeekdayFormat, locale?: string): string[];
}

/**
 * Day format values for DateFormatter.
 * @public
 */
export declare const DayFormat: {
    readonly "2-digit": "2-digit";
    readonly numeric: "numeric";
};

/**
 * Day format type for DateFormatter.
 * @public
 */
export declare type DayFormat = typeof DayFormat[keyof typeof DayFormat];

/**
 * Includes ARIA states and properties relating to the ARIA button role
 *
 * @public
 */
export declare class DelegatesARIAButton {
    /**
     * See {@link https://www.w3.org/WAI/PF/aria/roles#button} for more information
     * @public
     * @remarks
     * HTML Attribute: aria-expanded
     */
    ariaExpanded: "true" | "false" | string | null;
    /**
     * See {@link https://www.w3.org/WAI/PF/aria/roles#button} for more information
     * @public
     * @remarks
     * HTML Attribute: aria-pressed
     */
    ariaPressed: "true" | "false" | "mixed" | string | null;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface DelegatesARIAButton extends ARIAGlobalStatesAndProperties {
}

/**
 * Includes ARIA states and properties relating to the ARIA combobox role.
 *
 * @public
 */
export declare class DelegatesARIACombobox {
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#aria-autocomplete} for more information.
     *
     * @public
     * @remarks
     * HTML Attribute: `aria-autocomplete`
     */
    ariaAutoComplete: "inline" | "list" | "both" | "none" | string | null;
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#aria-controls} for more information.
     *
     * @public
     * @remarks
     * HTML Attribute: `aria-controls`
     */
    ariaControls: string | null;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface DelegatesARIACombobox extends DelegatesARIAListbox {
}

/**
 * Includes ARIA states and properties relating to the ARIA link role
 *
 * @public
 */
export declare class DelegatesARIALink {
    /**
     * See {@link https://www.w3.org/WAI/PF/aria/roles#link} for more information
     * @public
     * @remarks
     * HTML Attribute: aria-expanded
     */
    ariaExpanded: "true" | "false" | string | null;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface DelegatesARIALink extends ARIAGlobalStatesAndProperties {
}

/**
 * Includes ARIA states and properties relating to the ARIA listbox role
 *
 * @public
 */
export declare class DelegatesARIAListbox {
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#listbox} for more information
     * @public
     * @remarks
     * HTML Attribute: `aria-activedescendant`
     */
    ariaActiveDescendant: string | null;
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#listbox} for more information
     * @public
     * @remarks
     * HTML Attribute: `aria-disabled`
     */
    ariaDisabled: "true" | "false" | string | null;
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#listbox} for more information
     * @public
     * @remarks
     * HTML Attribute: `aria-expanded`
     */
    ariaExpanded: "true" | "false" | string | null;
    /**
     * See {@link https://w3c.github.io/aria/#listbox} for more information
     * @public
     * @remarks
     * HTML Attribute: `aria-multiselectable`
     */
    ariaMultiSelectable: "true" | "false" | string | null;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface DelegatesARIAListbox extends ARIAGlobalStatesAndProperties {
}

/**
 * States and properties relating to the ARIA `option` role.
 *
 * @public
 */
export declare class DelegatesARIAListboxOption {
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#option} for more information.
     * @public
     * @remarks
     * HTML Attribute: `aria-checked`
     */
    ariaChecked: "true" | "false" | string | null;
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#option} for more information.
     * @public
     * @remarks
     * HTML Attribute: `aria-posinset`
     */
    ariaPosInSet: string | null;
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#option} for more information.
     * @public
     * @remarks
     * HTML Attribute: `aria-selected`
     */
    ariaSelected: "true" | "false" | string | null;
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#option} for more information.
     * @public
     * @remarks
     * HTML Attribute: `aria-setsize`
     */
    ariaSetSize: string | null;
}

/**
 * @internal
 * @privateRemarks
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 */
export declare interface DelegatesARIAListboxOption extends ARIAGlobalStatesAndProperties {
}

/**
 * Includes ARIA states and properties relating to the ARIA textbox role
 *
 * @public
 */
export declare class DelegatesARIASearch {
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface DelegatesARIASearch extends ARIAGlobalStatesAndProperties {
}

/**
 * Includes ARIA states and properties relating to the ARIA select role.
 *
 * @public
 */
export declare class DelegatesARIASelect {
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#combobox} for more information
     * @public
     * @remarks
     * HTML Attribute: `aria-controls`
     */
    ariaControls: string | null;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface DelegatesARIASelect extends DelegatesARIAListbox {
}

/**
 * Includes ARIA states and properties relating to the ARIA textbox role
 *
 * @public
 */
export declare class DelegatesARIATextbox {
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface DelegatesARIATextbox extends ARIAGlobalStatesAndProperties {
}

/**
 * Includes ARIA states and properties relating to the ARIA toolbar role
 *
 * @public
 */
export declare class DelegatesARIAToolbar {
    /**
     * The id of the element labeling the toolbar.
     * @public
     * @remarks
     * HTML Attribute: aria-labelledby
     */
    ariaLabelledby: string | null;
    /**
     * The label surfaced to assistive technologies.
     *
     * @public
     * @remarks
     * HTML Attribute: aria-label
     */
    ariaLabel: string | null;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface DelegatesARIAToolbar extends ARIAGlobalStatesAndProperties {
}

/**
 * A {@link DesignToken} value that is derived. These values can depend on other {@link DesignToken}s
 * or arbitrary observable properties.
 * @public
 */
export declare type DerivedDesignTokenValue<T> = (resolve: DesignTokenResolver) => T;

/**
 * @public
 */
export declare class DesignToken<T> {
    /**
     * The name of the {@link DesignToken}
     */
    name: string;
    /**
     * The default value of the token (alias of {@link DesignToken.default})
     */
    get $value(): T | undefined;
    /**
     * The default value of the token, or undefined if it has not been set.
     */
    get default(): T | undefined;
    private _subscribers;
    private get subscribers();
    constructor(configuration: DesignTokenConfiguration);
    private static isCSSDesignTokenConfiguration;
    /**
     *
     * @param name - Factory function for creating a {@link DesignToken} or {@link CSSDesignToken}
     */
    static create<T>(name: string): CSSDesignToken<T>;
    static create<T>(config: DesignTokenConfiguration): DesignToken<T>;
    static create<T>(config: CSSDesignTokenConfiguration): CSSDesignToken<T>;
    /**
     * Configures the strategy for resolving hierarchical relationships between FASTElement targets.
     */
    static withStrategy(strategy: DesignTokenResolutionStrategy): void;
    /**
     * Registers a target for emitting default style values.
     * {@link CSSDesignToken | CSSDesignTokens} with default values assigned via
     * {@link DesignToken.withDefault} will emit CSS custom properties to all
     * registered targets.
     * @param target - The target to register, defaults to the document
     */
    static registerDefaultStyleTarget(target?: FASTElement | Document | PropertyTarget): void;
    /**
     * Unregister a target for default style emission.
     * @param target - The root to deregister, defaults to the document
     */
    static unregisterDefaultStyleTarget(target?: FASTElement | Document | PropertyTarget): void;
    /**
     * Retrieves the value of the token for a target element.
     */
    getValueFor(target: FASTElement): T;
    /**
     * Sets the value of the token for a target element.
     */
    setValueFor(target: FASTElement, value: DesignToken<T> | DesignTokenValue<T>): void;
    /**
     * Deletes the value of the token for a target element.
     */
    deleteValueFor(target: FASTElement): this;
    /**
     * Sets the default value of the token.
     */
    withDefault(value: DesignToken<T> | DesignTokenValue<T>): this;
    /**
     * Subscribes a subscriber to notifications for the token.
     */
    subscribe(subscriber: DesignTokenSubscriber<this>): void;
    /**
     * Unsubscribes a subscriber to notifications for the token.
     */
    unsubscribe(subscriber: DesignTokenSubscriber<this>): void;
    /**
     * Alias the token to the provided token.
     * @param token - the token to alias to
     */
    private alias;
    private normalizeValue;
    private subscriberNotifier;
}

/**
 * @public
 */
declare interface DesignToken_2<T> {
    readonly $value: T | undefined;
}

/**
 * @public
 */
export declare interface DesignTokenChangeRecord<T extends DesignToken<any>> {
    /**
     * The element for which the value was changed
     */
    target: FASTElement | "default";
    /**
     * The token that was changed
     */
    token: T;
}

/**
 * Describes a {@link DesignToken} configuration
 * @public
 */
export declare interface DesignTokenConfiguration {
    /**
     * The name of the {@link DesignToken}.
     */
    name: string;
}

/**
 * A DesignToken resolution strategy that uses custom events to resolve
 * node hierarchies.
 *
 * @public
 */
export declare const DesignTokenEventResolutionStrategy: DesignTokenResolutionStrategy;

/**
 * @public
 */
export declare const enum DesignTokenMutationType {
    add = 0,
    change = 1,
    delete = 2
}

declare interface DesignTokenResolutionStrategy extends HostBehavior<FASTElement> {
    /**
     * Determines if a 'child' element is contained by a 'parent'.
     * @param child - The child element
     * @param parent - The parent element
     */
    contains(parent: FASTElement, child: FASTElement): boolean;
    /**
     * Finds the nearest FASTElement parent node
     * @param element - The element to find the parent of
     */
    parent(element: FASTElement): FASTElement | null;
}

/**
 * A function that resolves the value of a DesignToken.
 * @public
 */
export declare type DesignTokenResolver = <T>(token: DesignToken_2<T>) => T;

/**
 * A constructable style target that can be registered
 * for DesignToken default style emission.
 *
 * Useful for controlling where CSS is emitted to, or when needing
 * to collect styles for SSR processes.
 *
 * @public
 */
export declare class DesignTokenStyleTarget implements PropertyTarget {
    private properties;
    setProperty(name: string, value: string): void;
    removeProperty(name: string): void;
    /**
     * The CSS text for the style target.
     * The text does *not* contain [CSS selector text](https://developer.mozilla.org/en-US/docs/Web/CSS/CSS_Selectors).
     */
    get cssText(): string;
    /**
     * The values set for the target as an array of key/value pairs.
     */
    get values(): Array<[string, string]>;
}

/**
 * A subscriber that should receive {@link DesignTokenChangeRecord | change records} when a token changes for a target
 * @public
 */
export declare interface DesignTokenSubscriber<T extends DesignToken<any>> {
    handleChange(token: T, record: DesignTokenChangeRecord<T>): void;
}

/**
 * The type that a {@link DesignToken} can be set to.
 * @public
 */
declare type DesignTokenValue<T> = StaticDesignTokenValue<T> | DerivedDesignTokenValue<T>;

/**
 * The template for the {@link @microsoft/fast-foundation#FASTDialog} component.
 * @public
 */
export declare function dialogTemplate<T extends FASTDialog>(): ElementViewTemplate<T>;

/**
 * @internal
 */
export declare interface Dimension {
    height: number;
    width: number;
}

/**
 * The CSS value for disabled cursors.
 * @public
 * @deprecated - fast-foundation is removing styling utilities
 */
export declare const disabledCursor = "not-allowed";

/**
 * The template for the {@link @microsoft/fast-foundation#FASTDisclosure} component.
 * @public
 */
export declare function disclosureTemplate<T extends FASTDisclosure>(): ElementViewTemplate<T>;

/**
 * Applies a CSS display property.
 * Also adds CSS rules to not display the element when the [hidden] attribute is applied to the element.
 * @param display - The CSS display property value
 * @public
 * @deprecated - fast-foundation is removing styling utilities
 */
export declare function display(displayValue: CSSDisplayPropertyValue): string;

/**
 * Divider roles
 * @public
 */
export declare const DividerRole: {
    /**
     * The divider semantically separates content
     */
    readonly separator: "separator";
    /**
     * The divider has no semantic value and is for visual presentation only.
     */
    readonly presentation: "presentation";
};

/**
 * The types for Divider roles
 * @public
 */
export declare type DividerRole = typeof DividerRole[keyof typeof DividerRole];

/**
 * The template for the {@link @microsoft/fast-foundation#FASTDivider} component.
 * @public
 */
export declare function dividerTemplate<T extends FASTDivider>(): ElementViewTemplate<T>;

/**
 * Source:
 * https://html.spec.whatwg.org/multipage/custom-elements.html#elementinternals
 */
declare interface ElementInternals_2 {
    /**
     * Returns the form owner of internals target element.
     */
    readonly form: HTMLFormElement | null;
    /**
     * Returns a NodeList of all the label elements that internals target element is associated with.
     */
    readonly labels: NodeList;
    /**
     * Returns the error message that would be shown to the user if internals target element was to be checked for validity.
     */
    readonly validationMessage: string;
    /**
     * Returns the ValidityState object for internals target element.
     */
    readonly validity: ValidityState;
    /**
     * Returns true if internals target element will be validated when the form is submitted; false otherwise.
     */
    readonly willValidate: boolean;
    /**
     * Returns true if internals target element has no validity problems; false otherwise. Fires an invalid event at the element in the latter case.
     */
    checkValidity(): boolean;
    /**
     * Returns true if internals target element has no validity problems; otherwise,
     * returns false, fires an invalid event at the element, and (if the event isn't canceled) reports the problem to the user.
     */
    reportValidity(): boolean;
    /**
     * Sets both the state and submission value of internals target element to value.
     *
     * While "null" isn't enumerated as a argument type (here)[https://html.spec.whatwg.org/multipage/custom-elements.html#the-elementinternals-interface],
     * In practice it appears to remove the value from the form data on submission. Adding it as a valid type here
     * becuase that capability is required for checkbox and radio types
     */
    setFormValue(value: File | string | FormData | null, state?: File | string | FormData | null): void;
    /**
     * Marks internals target element as suffering from the constraints indicated by the flags argument,
     * and sets the element's validation message to message.
     * If anchor is specified, the user agent might use
     * it to indicate problems with the constraints of internals target
     * element when the form owner is validated interactively or reportValidity() is called.
     */
    setValidity(flags: ValidityStateFlags, message?: string, anchor?: HTMLElement): void;
}

declare let ElementInternals_2: {
    prototype: ElementInternals_2;
    new (): ElementInternals_2;
};

/**
 * End configuration options
 * @public
 */
export declare type EndOptions = {
    end?: string | SyntheticViewTemplate;
};

/**
 * The template for the end slot.
 * For use with {@link StartEnd}
 *
 * @public
 */
export declare function endSlotTemplate(options: EndOptions): ViewTemplate<StartEnd>;

/**
 * An Accordion Custom HTML Element
 * Implements {@link https://www.w3.org/TR/wai-aria-practices-1.1/#accordion | ARIA Accordion}.
 *
 * @fires change - Fires a custom 'change' event when the active item changes
 * @csspart item - The slot for the accordion items
 * @public
 *
 * @remarks
 * Designed to be used with {@link @microsoft/fast-foundation#accordionTemplate} and {@link @microsoft/fast-foundation#(FASTAccordionItem:class)}.
 */
export declare class FASTAccordion extends FASTElement {
    /**
     * Controls the expand mode of the Accordion, either allowing
     * single or multiple item expansion.
     * @public
     *
     * @remarks
     * HTML attribute: expand-mode
     */
    expandmode: AccordionExpandMode;
    /**
     * @internal
     */
    accordionItems: HTMLElement[];
    /**
     * @internal
     */
    accordionItemsChanged(oldValue: HTMLElement[], newValue: HTMLElement[]): void;
    private activeid;
    private activeItemIndex;
    private accordionIds;
    private change;
    private findExpandedItem;
    private setItems;
    private resetItems;
    private removeItemListeners;
    private activeItemChange;
    private getItemIds;
    private isSingleExpandMode;
    private handleItemKeyDown;
    private handleItemFocus;
    private adjust;
    private focusItem;
}

/**
 * An individual item in an {@link @microsoft/fast-foundation#(FASTAccordion:class) }.
 *
 * @slot start - Content which can be provided between the heading and the icon
 * @slot end - Content which can be provided between the start slot and icon
 * @slot heading - Content which serves as the accordion item heading and text of the expand button
 * @slot - The default slot for accordion item content
 * @slot expanded-icon - The expanded icon
 * @slot collapsed-icon - The collapsed icon
 * @fires change - Fires a custom 'change' event when the button is invoked
 * @csspart heading - Wraps the button
 * @csspart button - The button which serves to invoke the item
 * @csspart heading-content - Wraps the slot for the heading content within the button
 * @csspart icon - The icon container
 * @csspart region - The wrapper for the accordion item content
 *
 * @public
 */
export declare class FASTAccordionItem extends FASTElement {
    /**
     * Configures the {@link https://www.w3.org/TR/wai-aria-1.1/#aria-level | level} of the
     * heading element.
     *
     * @defaultValue 2
     * @public
     * @remarks
     * HTML attribute: heading-level
     */
    headinglevel: 1 | 2 | 3 | 4 | 5 | 6;
    /**
     * Expands or collapses the item.
     *
     * @public
     * @remarks
     * HTML attribute: expanded
     */
    expanded: boolean;
    /**
     * The item ID
     *
     * @public
     * @remarks
     * HTML Attribute: id
     */
    id: string;
    /**
     * @internal
     */
    expandbutton: HTMLElement;
    /**
     * @internal
     */
    clickHandler: (e: MouseEvent) => void;
    private change;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface FASTAccordionItem extends StartEnd {
}

/**
 * An Anchor Custom HTML Element.
 * Based largely on the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element }.
 *
 * @slot start - Content which can be provided before the anchor content
 * @slot end - Content which can be provided after the anchor content
 * @slot - The default slot for anchor content
 * @csspart control - The anchor element
 * @csspart content - The element wrapping anchor content
 *
 * @public
 */
export declare class FASTAnchor extends FASTElement {
    /**
     * Prompts the user to save the linked URL. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: download
     */
    download: string;
    /**
     * The URL the hyperlink references. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: href
     */
    href: string;
    /**
     * Hints at the language of the referenced resource. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: hreflang
     */
    hreflang: string;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: ping
     */
    ping: string;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: referrerpolicy
     */
    referrerpolicy: string;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: rel
     */
    rel: string;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: target
     */
    target: AnchorTarget;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: type
     */
    type: string;
    /**
     *
     * Default slotted content
     *
     * @internal
     */
    defaultSlottedContent: HTMLElement[];
    /**
     * References the root element
     */
    control: HTMLAnchorElement;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * Overrides the focus call for where delegatesFocus is unsupported.
     * This check works for Chrome, Edge Chromium, FireFox, and Safari
     * Relevant PR on the Firefox browser: https://phabricator.services.mozilla.com/D123858
     */
    private handleUnsupportedDelegatesFocus;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface FASTAnchor extends StartEnd, DelegatesARIALink {
}

/**
 * An anchored region Custom HTML Element.
 *
 * @slot - The default slot for the content
 * @fires loaded - Fires a custom 'loaded' event when the region is loaded and visible
 * @fires positionchange - Fires a custom 'positionchange' event when the position has changed
 *
 * @public
 */
export declare class FASTAnchoredRegion extends FASTElement {
    /**
     * The HTML ID of the anchor element this region is positioned relative to
     *
     * @public
     * @remarks
     * HTML Attribute: anchor
     */
    anchor: string;
    protected anchorChanged(): void;
    /**
     * The HTML ID of the viewport element this region is positioned relative to
     *
     * @public
     * @remarks
     * HTML Attribute: anchor
     */
    viewport: string;
    protected viewportChanged(): void;
    /**
     * Sets what logic the component uses to determine horizontal placement.
     * 'locktodefault' forces the default position
     * 'dynamic' decides placement based on available space
     * 'uncontrolled' does not control placement on the horizontal axis
     *
     * @public
     * @remarks
     * HTML Attribute: horizontal-positioning-mode
     */
    horizontalPositioningMode: AxisPositioningMode;
    protected horizontalPositioningModeChanged(): void;
    /**
     * The default horizontal position of the region relative to the anchor element
     *
     * @public
     * @remarks
     * HTML Attribute: horizontal-default-position
     */
    horizontalDefaultPosition: HorizontalPosition;
    protected horizontalDefaultPositionChanged(): void;
    /**
     * Whether the region remains in the viewport (ie. detaches from the anchor) on the horizontal axis
     *
     * @public
     * @remarks
     * HTML Attribute: horizontal-viewport-lock
     */
    horizontalViewportLock: boolean;
    protected horizontalViewportLockChanged(): void;
    /**
     * Whether the region overlaps the anchor on the horizontal axis
     *
     * @public
     * @remarks
     * HTML Attribute: horizontal-inset
     */
    horizontalInset: boolean;
    protected horizontalInsetChanged(): void;
    /**
     * How narrow the space allocated to the default position has to be before the widest area
     * is selected for layout
     *
     * @public
     * @remarks
     * HTML Attribute: horizontal-threshold
     */
    horizontalThreshold: number;
    protected horizontalThresholdChanged(): void;
    /**
     * Defines how the width of the region is calculated
     *
     * @public
     * @remarks
     * HTML Attribute: horizontal-scaling
     */
    horizontalScaling: AxisScalingMode;
    protected horizontalScalingChanged(): void;
    /**
     * Sets what logic the component uses to determine vertical placement.
     * 'locktodefault' forces the default position
     * 'dynamic' decides placement based on available space
     * 'uncontrolled' does not control placement on the vertical axis
     *
     * @public
     * @remarks
     * HTML Attribute: vertical-positioning-mode
     */
    verticalPositioningMode: AxisPositioningMode;
    protected verticalPositioningModeChanged(): void;
    /**
     * The default vertical position of the region relative to the anchor element
     *
     * @public
     * @remarks
     * HTML Attribute: vertical-default-position
     */
    verticalDefaultPosition: VerticalPosition;
    protected verticalDefaultPositionChanged(): void;
    /**
     * Whether the region remains in the viewport (ie. detaches from the anchor) on the vertical axis
     *
     * @public
     * @remarks
     * HTML Attribute: vertical-viewport-lock
     */
    verticalViewportLock: boolean;
    protected verticalViewportLockChanged(): void;
    /**
     * Whether the region overlaps the anchor on the vertical axis
     *
     * @public
     * @remarks
     * HTML Attribute: vertical-inset
     */
    verticalInset: boolean;
    protected verticalInsetChanged(): void;
    /**
     * How short the space allocated to the default position has to be before the tallest area
     * is selected for layout
     *
     * @public
     * @remarks
     * HTML Attribute: vertical-threshold
     */
    verticalThreshold: number;
    protected verticalThresholdChanged(): void;
    /**
     * Defines how the height of the region is calculated
     *
     * @public
     * @remarks
     * HTML Attribute: vertical-scaling
     */
    verticalScaling: AxisScalingMode;
    protected verticalScalingChanged(): void;
    /**
     * Whether the region is positioned using css "position: fixed".
     * Otherwise the region uses "position: absolute".
     * Fixed placement allows the region to break out of parent containers,
     *
     * @public
     * @remarks
     * HTML Attribute: fixed-placement
     */
    fixedPlacement: boolean;
    protected fixedPlacementChanged(): void;
    /**
     * Defines what triggers the anchored region to revaluate positioning
     *
     * @public
     * @remarks
     * HTML Attribute: auto-update-mode
     */
    autoUpdateMode: AutoUpdateMode;
    protected autoUpdateModeChanged(prevMode: AutoUpdateMode, newMode: AutoUpdateMode): void;
    /**
     * The HTML element being used as the anchor
     *
     * @public
     */
    anchorElement: HTMLElement | null;
    protected anchorElementChanged(): void;
    /**
     * The HTML element being used as the viewport
     *
     * @public
     */
    viewportElement: HTMLElement | null;
    protected viewportElementChanged(): void;
    /**
     * indicates that an initial positioning pass on layout has completed
     *
     * @internal
     */
    initialLayoutComplete: boolean;
    /**
     * indicates the current horizontal position of the region
     */
    verticalPosition: AnchoredRegionPositionLabel | undefined;
    /**
     * indicates the current vertical position of the region
     */
    horizontalPosition: AnchoredRegionPositionLabel | undefined;
    /**
     * values to be applied to the component's transform on render
     */
    private translateX;
    private translateY;
    /**
     * the span to be applied to the region on each axis
     */
    private regionWidth;
    private regionHeight;
    private resizeDetector;
    private viewportRect;
    private anchorRect;
    private regionRect;
    /**
     * base offsets between the positioner's base position and the anchor's
     */
    private baseHorizontalOffset;
    private baseVerticalOffset;
    private pendingPositioningUpdate;
    private pendingReset;
    private currentDirection;
    private regionVisible;
    private forceUpdate;
    private updateThreshold;
    private static intersectionService;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * @internal
     */
    disconnectedCallback(): void;
    /**
     * @internal
     */
    adoptedCallback(): void;
    /**
     * update position
     */
    update: () => void;
    /**
     * destroys the instance's resize observer
     */
    private disconnectResizeDetector;
    /**
     * initializes the instance's resize observer
     */
    private initializeResizeDetector;
    /**
     * react to attribute changes that don't require a reset
     */
    private updateForAttributeChange;
    /**
     * fully initializes the component
     */
    private initialize;
    /**
     * Request a reset if there are currently no open requests
     */
    private requestReset;
    /**
     * sets the starting configuration for component internal values
     */
    private setInitialState;
    /**
     * starts observers
     */
    private startObservers;
    /**
     * get position updates
     */
    private requestPositionUpdates;
    /**
     * stops observers
     */
    private stopObservers;
    /**
     * Gets the viewport element by id, or defaults to document root
     */
    private getViewport;
    /**
     *  Gets the anchor element by id
     */
    private getAnchor;
    /**
     *  Handle intersections
     */
    private handleIntersection;
    /**
     *  iterate through intersection entries and apply data
     */
    private applyIntersectionEntries;
    /**
     *  Update the offset values
     */
    private updateRegionOffset;
    /**
     *  compare rects to see if there is enough change to justify a DOM update
     */
    private isRectDifferent;
    /**
     *  Handle resize events
     */
    private handleResize;
    /**
     * resets the component
     */
    private reset;
    /**
     *  Recalculate layout related state values
     */
    private updateLayout;
    /**
     *  Updates the style string applied to the region element as well as the css classes attached
     *  to the root element
     */
    private updateRegionStyle;
    /**
     *  Updates the css classes that reflect the current position of the element
     */
    private updatePositionClasses;
    /**
     * Get horizontal positioning state based on desired position
     */
    private setHorizontalPosition;
    /**
     * Set vertical positioning state based on desired position
     */
    private setVerticalPosition;
    /**
     *  Get available positions based on positioning mode
     */
    private getPositioningOptions;
    /**
     *  Get the space available for a particular relative position
     */
    private getAvailableSpace;
    /**
     * Get region dimensions
     */
    private getNextRegionDimension;
    /**
     * starts event listeners that can trigger auto updating
     */
    private startAutoUpdateEventListeners;
    /**
     * stops event listeners that can trigger auto updating
     */
    private stopAutoUpdateEventListeners;
}

/**
 * An Avatar Custom HTML Element
 *
 * @slot media - Used for media such as an image
 * @slot - The default slot for avatar text, commonly a name or initials
 * @slot badge - Used to provide a badge, such as a status badge
 * @csspart backplate - The wrapping container for the avatar
 * @csspart link - The avatar link
 * @csspart content - The default slot
 *
 * @public
 */
export declare class FASTAvatar extends FASTElement {
    /**
     * Indicates the Avatar should have url link
     *
     * @public
     * @remarks
     * HTML Attribute: link
     */
    link: string;
}

/**
 * A Badge Custom HTML Element.
 * @slot - The default slot for the badge
 * @csspart control - The element representing the badge, which wraps the default slot
 *
 * @public
 */
export declare class FASTBadge extends FASTElement {
}

/**
 * A base class for progress components.
 * @public
 */
export declare class FASTBaseProgress extends FASTElement {
    /**
     * The value of the progress
     * @public
     * @remarks
     * HTML Attribute: value
     */
    value: number | null;
    protected valueChanged(): void;
    /**
     * The minimum value
     * @public
     * @remarks
     * HTML Attribute: min
     */
    min: number;
    protected minChanged(): void;
    /**
     * The maximum value
     * @public
     * @remarks
     * HTML Attribute: max
     */
    max: number;
    protected maxChanged(): void;
    /**
     * Indicates the progress is paused
     * @public
     * @remarks
     * HTML Attribute: paused
     */
    paused: boolean;
    /**
     * Indicates progress in %
     * @internal
     */
    percentComplete: number;
    /**
     * @internal
     */
    connectedCallback(): void;
    private updatePercentComplete;
}

/**
 * A Breadcrumb Custom HTML Element.
 * @slot - The default slot for the breadcrumb items
 * @csspart list - The element wrapping the slotted items
 *
 * @public
 */
export declare class FASTBreadcrumb extends FASTElement {
    /**
     * @internal
     */
    slottedBreadcrumbItems: HTMLElement[];
    protected slottedBreadcrumbItemsChanged(): void;
    private setItemSeparator;
    /**
     * Finds anchor childnodes in the light DOM or shadow DOM.
     * We look in the shadow DOM because we use an anchor inside the breadcrumb-item template.
     */
    private findChildAnchor;
    /**
     * Sets ARIA Current for the "current" node
     * `aria-current` is not optional and should be set regardless of the href value of a given anchor
     */
    private setAriaCurrent;
}

/**
 * A Breadcrumb Item Custom HTML Element.
 *
 * @public
 */
export declare class FASTBreadcrumbItem extends FASTAnchor {
    /**
     * @internal
     */
    separator: boolean;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface FASTBreadcrumbItem extends StartEnd, DelegatesARIALink {
}

/**
 * A Button Custom HTML Element.
 * Based largely on the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button | <button> element }.
 *
 * @slot start - Content which can be provided before the button content
 * @slot end - Content which can be provided after the button content
 * @slot - The default slot for button content
 * @csspart control - The button element
 * @csspart content - The element wrapping button content
 *
 * @public
 */
export declare class FASTButton extends FormAssociatedButton {
    /**
     * Determines if the element should receive document focus on page load.
     *
     * @public
     * @remarks
     * HTML Attribute: autofocus
     */
    autofocus: boolean;
    /**
     * The id of a form to associate the element to.
     *
     * @public
     * @remarks
     * HTML Attribute: form
     */
    formId: string;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button | <button> element} for more details.
     *
     * @public
     * @remarks
     * HTML Attribute: formaction
     */
    formaction: string;
    protected formactionChanged(): void;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button | <button> element} for more details.
     *
     * @public
     * @remarks
     * HTML Attribute: formenctype
     */
    formenctype: string;
    protected formenctypeChanged(): void;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button | <button> element} for more details.
     *
     * @public
     * @remarks
     * HTML Attribute: formmethod
     */
    formmethod: string;
    protected formmethodChanged(): void;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button | <button> element} for more details.
     *
     * @public
     * @remarks
     * HTML Attribute: formnovalidate
     */
    formnovalidate: boolean;
    protected formnovalidateChanged(): void;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button | <button> element} for more details.
     *
     * @public
     * @remarks
     * HTML Attribute: formtarget
     */
    formtarget: "_self" | "_blank" | "_parent" | "_top";
    protected formtargetChanged(): void;
    /**
     * The button type.
     *
     * @public
     * @remarks
     * HTML Attribute: type
     */
    type: ButtonType;
    protected typeChanged(previous: ButtonType | undefined, next: ButtonType): void;
    /**
     *
     * Default slotted content
     *
     * @public
     * @remarks
     */
    defaultSlottedContent: HTMLElement[];
    /** {@inheritDoc (FormAssociated:interface).validate} */
    validate(): void;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * Submits the parent form
     */
    private handleSubmission;
    /**
     * Resets the parent form
     */
    private handleFormReset;
    control: HTMLButtonElement;
    /**
     * Overrides the focus call for where delegatesFocus is unsupported.
     * This check works for Chrome, Edge Chromium, FireFox, and Safari
     * Relevant PR on the Firefox browser: https://phabricator.services.mozilla.com/D123858
     */
    private handleUnsupportedDelegatesFocus;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface FASTButton extends StartEnd, DelegatesARIAButton {
}

/**
 * Calendar component
 *
 * @slot - The default slot for calendar content
 * @fires dateselected - Fires a custom 'dateselected' event when Enter is invoked via keyboard on a date
 *
 * @public
 */
export declare class FASTCalendar extends FASTElement {
    /**
     * date formatter utitlity for getting localized strings
     * @public
     */
    dateFormatter: DateFormatter;
    /**
     * Readonly attribute for turning off data-grid
     * @public
     */
    readonly: boolean;
    /**
     * String repesentation of the full locale including market, calendar type and numbering system
     * @public
     */
    locale: string;
    protected localeChanged(): void;
    /**
     * Weekday that the calendar should start on, defaults to Sunday
     * @public
     */
    firstDay: number;
    /**
     * Month to display
     * @public
     */
    month: number;
    /**
     * Year of the month to display
     * @public
     */
    year: number;
    /**
     * Format style for the day
     * @public
     */
    dayFormat: DayFormat;
    protected dayFormatChanged(): void;
    /**
     * Format style for the week day labels
     * @public
     */
    weekdayFormat: WeekdayFormat;
    protected weekdayFormatChanged(): void;
    /**
     * Format style for the month label
     * @public
     */
    monthFormat: MonthFormat;
    protected monthFormatChanged(): void;
    /**
     * Format style for the year used in the title
     * @public
     */
    yearFormat: YearFormat;
    protected yearFormatChanged(): void;
    /**
     * Minimum number of weeks to show for the month
     * This can be used to normalize the calendar view
     *  when changing or across multiple calendars
     * @public
     */
    minWeeks: number;
    /**
     * A list of dates that should be shown as disabled
     * @public
     */
    disabledDates: string;
    /**
     * A list of dates that should be shown as highlighted
     * @public
     */
    selectedDates: string;
    /**
     * The number of miliseconds in a day
     * @internal
     */
    private oneDayInMs;
    /**
     * Gets data needed to render about a calendar month as well as the previous and next months
     * @param year - year of the calendar
     * @param month - month of the calendar
     * @returns - an object with data about the current and 2 surrounding months
     * @public
     */
    getMonthInfo(month?: number, year?: number): CalendarInfo;
    /**
     * A list of calendar days
     * @param info - an object containing the information needed to render a calendar month
     * @param minWeeks - minimum number of weeks to show
     * @returns a list of days in a calendar month
     * @public
     */
    getDays(info?: CalendarInfo, minWeeks?: number): CalendarDateInfo[][];
    /**
     * A helper function that checks if a date exists in a list of dates
     * @param date - A date objec that includes the day, month and year
     * @param datesString - a comma separated list of dates
     * @returns - Returns true if it found the date in the list of dates
     * @public
     */
    dateInString(date: Date | string, datesString: string): boolean;
    /**
     * Creates a class string for the day container
     * @param date - date of the calendar cell
     * @returns - string of class names
     * @public
     */
    getDayClassNames(date: CalendarDateInfo, todayString?: string): string;
    /**
     * Returns a list of weekday labels
     * @returns An array of weekday text and full text if abbreviated
     * @public
     */
    getWeekdayText(): WeekdayText[];
    /**
     * Emits the "date-select" event with the day, month and year.
     * @param date - Date cell
     * @public
     */
    handleDateSelect(event: Event, day: CalendarDateInfo): void;
    /**
     * Handles keyboard events on a cell
     * @param event - Keyboard event
     * @param date - Date of the cell selected
     */
    handleKeydown(event: KeyboardEvent, date: CalendarDateInfo): boolean;
}

/**
 * An Card Custom HTML Element.
 *
 * @slot - The default slot for the card content
 *
 * @public
 */
export declare class FASTCard extends FASTElement {
}

/**
 * A Checkbox Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#checkbox | ARIA checkbox }.
 *
 * @slot checked-indicator - The checked indicator
 * @slot indeterminate-indicator - The indeterminate indicator
 * @slot - The default slot for the label
 * @csspart control - The element representing the visual checkbox control
 * @csspart label - The label
 * @fires change - Emits a custom change event when the checked state changes
 *
 * @public
 */
export declare class FASTCheckbox extends FormAssociatedCheckbox {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /**
     * The element's value to be included in form submission when checked.
     * Default to "on" to reach parity with input[type="checkbox"]
     *
     * @internal
     */
    initialValue: string;
    /**
     * @internal
     */
    defaultSlottedNodes: Node[];
    /**
     * The indeterminate state of the control
     */
    indeterminate: boolean;
    constructor();
    /**
     * @internal
     */
    keypressHandler: (e: KeyboardEvent) => void;
    /**
     * @internal
     */
    clickHandler: (e: MouseEvent) => void;
}

/**
 * A Combobox Custom HTML Element.
 * Implements the {@link https://w3c.github.io/aria-practices/#combobox | ARIA combobox }.
 *
 * @slot start - Content which can be provided before the input
 * @slot end - Content which can be provided after the input
 * @slot control - Used to replace the input element representing the combobox
 * @slot indicator - The visual indicator representing the expanded state
 * @slot - The default slot for the options
 * @csspart control - The wrapper element containing the input area, including start and end
 * @csspart selected-value - The input element representing the selected value
 * @csspart indicator - The element wrapping the indicator slot
 * @csspart listbox - The wrapper for the listbox slotted options
 * @fires change - Fires a custom 'change' event when the value updates
 *
 * @public
 */
export declare class FASTCombobox extends FormAssociatedCombobox {
    /**
     * The internal value property.
     *
     * @internal
     */
    private _value;
    /**
     * The autocomplete attribute.
     *
     * @public
     * @remarks
     * HTML Attribute: autocomplete
     */
    autocomplete: ComboboxAutocomplete | undefined;
    /**
     * Reference to the internal text input element.
     *
     * @internal
     */
    control: HTMLInputElement;
    /**
     * Reference to the internal listbox element.
     *
     * @internal
     */
    listbox: HTMLDivElement;
    /**
     * The collection of currently filtered options.
     *
     * @public
     */
    filteredOptions: FASTListboxOption[];
    /**
     * The current filter value.
     *
     * @internal
     */
    private filter;
    /**
     * Reset the element to its first selectable option when its parent form is reset.
     *
     * @internal
     */
    formResetCallback(): void;
    /** {@inheritDoc (FormAssociated:interface).validate} */
    validate(): void;
    private get isAutocompleteInline();
    private get isAutocompleteList();
    private get isAutocompleteBoth();
    /**
     * The unique id for the internal listbox element.
     *
     * @internal
     */
    listboxId: string;
    /**
     * The open attribute.
     *
     * @public
     * @remarks
     * HTML Attribute: open
     */
    open: boolean;
    /**
     * Sets focus and synchronize ARIA attributes when the open property changes.
     *
     * @param prev - the previous open value
     * @param next - the current open value
     *
     * @internal
     */
    protected openChanged(): void;
    /**
     * The list of options.
     *
     * @public
     * @remarks
     * Overrides `Listbox.options`.
     */
    get options(): FASTListboxOption[];
    set options(value: FASTListboxOption[]);
    /**
     * Sets the placeholder value of the element, generally used to provide a hint to the user.
     * @public
     * @remarks
     * HTML Attribute: placeholder
     * Using this attribute is not a valid substitute for a labeling element.
     */
    placeholder: string;
    /**
     * Updates the placeholder on the proxy element.
     * @internal
     */
    protected placeholderChanged(): void;
    /**
     * The value property.
     *
     * @public
     */
    get value(): string;
    set value(next: string);
    /**
     * Cleanup function for the listbox positioner.
     *
     * @public
     */
    cleanup: () => void;
    /**
     * Handle opening and closing the listbox when the combobox is clicked.
     *
     * @param e - the mouse event
     * @internal
     */
    clickHandler(e: MouseEvent): boolean | void;
    connectedCallback(): void;
    /**
     * Synchronize the `aria-disabled` property when the `disabled` property changes.
     *
     * @param prev - The previous disabled value
     * @param next - The next disabled value
     *
     * @internal
     */
    disabledChanged(prev: boolean, next: boolean): void;
    disconnectedCallback(): void;
    /**
     * Filter available options by text value.
     *
     * @public
     */
    filterOptions(): void;
    /**
     * Focus the control and scroll the first selected option into view.
     *
     * @internal
     * @remarks
     * Overrides: `Listbox.focusAndScrollOptionIntoView`
     */
    protected focusAndScrollOptionIntoView(): void;
    /**
     * Handle focus state when the element or its children lose focus.
     *
     * @param e - The focus event
     * @internal
     */
    focusoutHandler(e: FocusEvent): boolean | void;
    /**
     * Handle content changes on the control input.
     *
     * @param e - the input event
     * @internal
     */
    inputHandler(e: InputEvent): boolean | void;
    /**
     * Handle keydown actions for listbox navigation.
     *
     * @param e - the keyboard event
     * @internal
     */
    keydownHandler(e: Event & KeyboardEvent): boolean | void;
    /**
     * Handle keyup actions for value input and text field manipulations.
     *
     * @param e - the keyboard event
     * @internal
     */
    keyupHandler(e: KeyboardEvent): boolean | void;
    /**
     * Ensure that the selectedIndex is within the current allowable filtered range.
     *
     * @param prev - the previous selected index value
     * @param next - the current selected index value
     *
     * @internal
     */
    selectedIndexChanged(prev: number | undefined, next: number): void;
    /**
     * Move focus to the previous selectable option.
     *
     * @internal
     * @remarks
     * Overrides `Listbox.selectPreviousOption`
     */
    selectPreviousOption(): void;
    /**
     * Set the default selected options at initialization or reset.
     *
     * @internal
     * @remarks
     * Overrides `Listbox.setDefaultSelectedOption`
     */
    setDefaultSelectedOption(): void;
    /**
     * Focus and select the content of the control based on the first selected option.
     *
     * @param start - The index for the starting range
     * @internal
     */
    private setInlineSelection;
    /**
     * Calculate and apply listbox positioning based on available viewport space.
     *
     * @public
     */
    setPositioning(): void;
    /**
     * Ensure that the entire list of options is used when setting the selected property.
     *
     * @param prev - the previous list of selected options
     * @param next - the current list of selected options
     *
     * @internal
     * @remarks
     * Overrides: `Listbox.selectedOptionsChanged`
     */
    selectedOptionsChanged(prev: FASTListboxOption[] | undefined, next: FASTListboxOption[]): void;
    /**
     * Synchronize the form-associated proxy and update the value property of the element.
     *
     * @param prev - the previous collection of slotted option elements
     * @param next - the next collection of slotted option elements
     *
     * @internal
     */
    slottedOptionsChanged(prev: Element[] | undefined, next: Element[]): void;
    /**
     * Sets the value and to match the first selected option.
     *
     * @param shouldEmit - if true, the change event will be emitted
     *
     * @internal
     */
    private updateValue;
    /**
     * @internal
     */
    private clearSelectionRange;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface FASTCombobox extends StartEnd, DelegatesARIACombobox {
}

/**
 * A Data Grid Custom HTML Element.
 *
 * @slot - The default slot for custom row elements
 * @public
 */
export declare class FASTDataGrid extends FASTElement {
    /**
     *  generates a basic column definition by examining sample row data
     */
    static generateColumns: (row: object) => ColumnDefinition[];
    /**
     *  generates a gridTemplateColumns based on columndata array
     */
    private static generateTemplateColumns;
    /**
     * When true the component will not add itself to the tab queue.
     * Default is false.
     *
     * @public
     * @remarks
     * HTML Attribute: no-tabbing
     */
    noTabbing: boolean;
    protected noTabbingChanged(): void;
    /**
     *  Whether the grid should automatically generate a header row and its type
     *
     * @public
     * @remarks
     * HTML Attribute: generate-header
     */
    generateHeader: GenerateHeaderOptions;
    private generateHeaderChanged;
    /**
     * String that gets applied to the the css gridTemplateColumns attribute of child rows
     *
     * @public
     * @remarks
     * HTML Attribute: grid-template-columns
     */
    gridTemplateColumns: string;
    protected gridTemplateColumnsChanged(): void;
    /**
     * The data being displayed in the grid
     *
     * @public
     */
    rowsData: object[];
    protected rowsDataChanged(): void;
    /**
     * The column definitions of the grid
     *
     * @public
     */
    columnDefinitions: ColumnDefinition[] | null;
    protected columnDefinitionsChanged(): void;
    /**
     * The template to use for the programmatic generation of rows
     *
     * @public
     */
    rowItemTemplate: ViewTemplate;
    /**
     * The template used to render cells in generated rows.
     *
     * @public
     */
    cellItemTemplate?: ViewTemplate;
    /**
     * The template used to render header cells in generated rows.
     *
     * @public
     */
    headerCellItemTemplate?: ViewTemplate;
    private headerCellItemTemplateChanged;
    /**
     * The index of the row that will receive focus the next time the
     * grid is focused. This value changes as focus moves to different
     * rows within the grid.  Changing this value when focus is already
     * within the grid moves focus to the specified row.
     *
     * @public
     */
    focusRowIndex: number;
    private focusRowIndexChanged;
    /**
     * The index of the column that will receive focus the next time the
     * grid is focused. This value changes as focus moves to different rows
     * within the grid.  Changing this value when focus is already within
     * the grid moves focus to the specified column.
     *
     * @public
     */
    focusColumnIndex: number;
    private focusColumnIndexChanged;
    /**
     * The default row item template.  Set by the component templates.
     *
     * @internal
     */
    defaultRowItemTemplate: ViewTemplate;
    /**
     * Set by the component templates.
     *
     */
    rowElementTag: string;
    /**
     * Children that are rows
     *
     * @internal
     */
    rowElements: HTMLElement[];
    private rowsPlaceholder;
    private behaviorOrchestrator;
    private generatedHeader;
    private isUpdatingFocus;
    private pendingFocusUpdate;
    private observer;
    private rowindexUpdateQueued;
    private columnDefinitionsStale;
    private generatedGridTemplateColumns;
    constructor();
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * @internal
     */
    disconnectedCallback(): void;
    /**
     * @internal
     */
    handleRowFocus(e: Event): void;
    /**
     * @internal
     */
    handleFocus(e: FocusEvent): void;
    /**
     * @internal
     */
    handleFocusOut(e: FocusEvent): void;
    /**
     * @internal
     */
    handleKeydown(e: KeyboardEvent): void;
    private focusOnCell;
    private queueFocusUpdate;
    private updateFocus;
    private toggleGeneratedHeader;
    private onChildListChange;
    private queueRowIndexUpdate;
    private updateRowIndexes;
}

/**
 * A Data Grid Cell Custom HTML Element.
 *
 * @fires cell-focused - Fires a custom 'cell-focused' event when focus is on the cell or its contents
 * @slot - The default slot for cell contents.  The "cell contents template" renders here.
 * @public
 */
export declare class FASTDataGridCell extends FASTElement {
    /**
     * The type of cell
     *
     * @public
     * @remarks
     * HTML Attribute: cell-type
     */
    cellType: DataGridCellTypes;
    private cellTypeChanged;
    /**
     * The column index of the cell.
     * This will be applied to the css grid-column-index value
     * applied to the cell
     *
     * @public
     * @remarks
     * HTML Attribute: grid-column
     */
    gridColumn: string;
    protected gridColumnChanged(): void;
    /**
     * The base data for the parent row
     *
     * @public
     */
    rowData: object | null;
    /**
     * The base data for the column
     *
     * @public
     */
    columnDefinition: ColumnDefinition | null;
    protected columnDefinitionChanged(oldValue: ColumnDefinition | null, newValue: ColumnDefinition | null): void;
    private isActiveCell;
    private customCellView;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * @internal
     */
    disconnectedCallback(): void;
    handleFocusin(e: FocusEvent): void;
    handleFocusout(e: FocusEvent): void;
    handleKeydown(e: KeyboardEvent): void;
    private updateCellView;
    private disconnectCellView;
    private updateCellStyle;
}

/**
 * A Data Grid Row Custom HTML Element.
 *
 * @fires row-focused - Fires a custom 'row-focused' event when focus is on an element (usually a cell or its contents) in the row
 * @slot - The default slot for custom cell elements
 * @public
 */
export declare class FASTDataGridRow extends FASTElement {
    /**
     * String that gets applied to the the css gridTemplateColumns attribute for the row
     *
     * @public
     * @remarks
     * HTML Attribute: grid-template-columns
     */
    gridTemplateColumns: string;
    protected gridTemplateColumnsChanged(): void;
    /**
     * The type of row
     *
     * @public
     * @remarks
     * HTML Attribute: row-type
     */
    rowType: DataGridRowTypes;
    private rowTypeChanged;
    /**
     * The base data for this row
     *
     * @public
     */
    rowData: object | null;
    protected rowDataChanged(): void;
    /**
     * The column definitions of the row
     *
     * @public
     */
    columnDefinitions: ColumnDefinition[] | null;
    /**
     * The template used to render cells in generated rows.
     *
     * @public
     */
    cellItemTemplate?: ViewTemplate;
    private cellItemTemplateChanged;
    /**
     * The template used to render header cells in generated rows.
     *
     * @public
     */
    headerCellItemTemplate?: ViewTemplate;
    private headerCellItemTemplateChanged;
    /**
     * The index of the row in the parent grid.
     * This is typically set programmatically by the parent grid.
     *
     * @public
     */
    rowIndex: number;
    /**
     * Whether focus is on/in a cell within this row.
     *
     * @internal
     */
    isActiveRow: boolean;
    /**
     * The cell item template currently in use.
     *
     * @internal
     */
    activeCellItemTemplate?: ViewTemplate;
    /**
     * The default cell item template.  Set by the component templates.
     *
     * @internal
     */
    defaultCellItemTemplate?: ViewTemplate;
    /**
     * The default header cell item template.  Set by the component templates.
     *
     * @internal
     */
    defaultHeaderCellItemTemplate?: ViewTemplate;
    /**
     * Children that are cells
     *
     * @internal
     */
    cellElements: HTMLElement[];
    private behaviorOrchestrator;
    /**
     * @internal
     */
    slottedCellElements: HTMLElement[];
    /**
     * @internal
     */
    focusColumnIndex: number;
    private refocusOnLoad;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * @internal
     */
    disconnectedCallback(): void;
    handleFocusout(e: FocusEvent): void;
    handleCellFocus(e: Event): void;
    handleKeydown(e: KeyboardEvent): void;
    private updateItemTemplate;
    private updateRowStyle;
}

/**
 * A Switch Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#dialog | ARIA dialog }.
 *
 * @slot - The default slot for the dialog content
 * @csspart positioning-region - A wrapping element used to center the dialog and position the modal overlay
 * @csspart overlay - The modal dialog overlay
 * @csspart control - The dialog element
 * @fires cancel - Fires a custom 'cancel' event when the modal overlay is clicked
 * @fires close - Fires a custom 'close' event when the dialog is hidden
 *
 * @public
 */
export declare class FASTDialog extends FASTElement {
    /**
     * Indicates the element is modal. When modal, user mouse interaction will be limited to the contents of the element by a modal
     * overlay.  Clicks on the overlay will cause the dialog to emit a "dismiss" event.
     * @public
     * @defaultValue - false
     * @remarks
     * HTML Attribute: modal
     */
    modal: boolean;
    /**
     * The hidden state of the element.
     *
     * @public
     * @defaultValue - false
     * @remarks
     * HTML Attribute: hidden
     */
    hidden: boolean;
    /**
     * Indicates that the dialog should not trap focus.
     *
     * @public
     * @defaultValue - true
     * @remarks
     * HTML Attribute: no-focus-trap
     */
    noFocusTrap: boolean;
    private noFocusTrapChanged;
    /**
     * The id of the element describing the dialog.
     * @public
     * @remarks
     * HTML Attribute: aria-describedby
     */
    ariaDescribedby: string;
    /**
     * The id of the element labeling the dialog.
     * @public
     * @remarks
     * HTML Attribute: aria-labelledby
     */
    ariaLabelledby: string;
    /**
     * The label surfaced to assistive technologies.
     *
     * @public
     * @remarks
     * HTML Attribute: aria-label
     */
    ariaLabel: string;
    /**
     * @internal
     */
    dialog: HTMLDivElement;
    /**
     * @internal
     */
    private isTrappingFocus;
    /**
     * @internal
     */
    private notifier;
    /**
     * @internal
     */
    dismiss(): void;
    /**
     * The method to show the dialog.
     *
     * @public
     */
    show(): void;
    /**
     * The method to hide the dialog.
     *
     * @public
     */
    hide(): void;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * @internal
     */
    disconnectedCallback(): void;
    /**
     * @internal
     */
    handleChange(source: any, propertyName: string): void;
    private handleDocumentKeydown;
    private handleDocumentFocus;
    private handleTabKeyDown;
    private getTabQueueBounds;
    /**
     * focus on first element of tab queue
     */
    private focusFirstElement;
    /**
     * we should only focus if focus has not already been brought to the dialog
     */
    private shouldForceFocus;
    /**
     * we should we be active trapping focus
     */
    private shouldTrapFocus;
    /**
     *
     *
     * @internal
     */
    private updateTrapFocus;
    /**
     * Reduce a collection to only its focusable elements.
     *
     * @param elements - Collection of elements to reduce
     * @param element - The current element
     *
     * @internal
     */
    private static reduceTabbableItems;
    /**
     * Test if element is focusable fast element
     *
     * @param element - The element to check
     *
     * @internal
     */
    private static isFocusableFastElement;
    /**
     * Test if the element has a focusable shadow
     *
     * @param element - The element to check
     *
     * @internal
     */
    private static hasTabbableShadow;
}

/**
 * A Disclosure Custom HTML Element.
 * Based largely on the {@link https://w3c.github.io/aria-practices/#disclosure | disclosure element }.
 *
 * @slot start - Content which can be provided before the summary content
 * @slot end - Content which can be provided after the summary content
 * @slot title - The summary content
 * @slot - The default slot for the disclosure content
 * @fires toggle - fires a toggle event when the summary is toggled
 *
 * @public
 */
export declare class FASTDisclosure extends FASTElement {
    /**
     * Determines if the element should show the extra content or not.
     *
     * @public
     */
    expanded: boolean;
    /**
     * Invoker title
     *
     * @public
     */
    summary: string;
    /**
     * @internal
     */
    details: HTMLDetailsElement;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * @internal
     */
    disconnectedCallback(): void;
    /**
     * Show extra content.
     */
    show(): void;
    /**
     * Hide extra content.
     */
    hide(): void;
    /**
     * Toggle the current(expanded/collapsed) state.
     */
    toggle(): void;
    /**
     * Register listener and set default disclosure mode
     */
    protected setup(): void;
    /**
     * Update the aria attr and fire `toggle` event
     */
    protected onToggle(): void;
}

/**
 * A Divider Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#separator | ARIA separator } or {@link https://www.w3.org/TR/wai-aria-1.1/#presentation | ARIA presentation}.
 *
 * @public
 */
export declare class FASTDivider extends FASTElement {
    /**
     * The role of the element.
     *
     * @public
     * @remarks
     * HTML Attribute: role
     */
    role: DividerRole;
    /**
     * The orientation of the divider.
     *
     * @public
     * @remarks
     * HTML Attribute: orientation
     */
    orientation: Orientation;
}

/**
 * A Flipper Custom HTML Element.
 * Flippers are a form of button that implies directional content navigation, such as in a carousel.
 *
 * @slot next - The next flipper content
 * @slot previous - The previous flipper content
 * @csspart next - Wraps the next flipper content
 * @csspart previous - Wraps the previous flipper content
 * @fires click - Fires a custom 'click' event when Enter or Space is invoked via keyboard and the flipper is exposed to assistive technologies.
 *
 * @public
 */
export declare class FASTFlipper extends FASTElement {
    /**
     * The disabled state of the flipper.
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
    /**
     * Indicates the flipper should be hidden from assistive technology. Because flippers are often supplementary navigation, they are often hidden from assistive technology.
     *
     * @public
     * @defaultValue - true
     * @remarks
     * HTML Attribute: aria-hidden
     */
    hiddenFromAT: boolean;
    /**
     * The direction that the flipper implies navigating.
     *
     * @public
     * @remarks
     * HTML Attribute: direction
     */
    direction: FlipperDirection;
    /**
     * Simulate a click event when the flipper has focus and the user hits enter or space keys
     * Blur focus if the user hits escape key
     * @param e - Keyboard event
     * @public
     */
    keyupHandler(e: Event & KeyboardEvent): void;
}

/**
 * A HorizontalScroll Custom HTML Element
 *
 * @slot start - Content which can be provided before the scroll area
 * @slot end - Content which can be provided after the scroll area
 * @csspart scroll-area - Wraps the entire scrollable region
 * @csspart scroll-view - The visible scroll area
 * @csspart content-container - The container for the content
 * @csspart scroll-prev - The previous flipper container
 * @csspart scroll-action-previous - The element wrapping the previous flipper
 * @csspart scroll-next - The next flipper container
 * @csspart scroll-action-next - The element wrapping the next flipper
 * @fires scrollstart - Fires a custom 'scrollstart' event when scrolling
 * @fires scrollend - Fires a custom 'scrollend' event when scrolling stops
 *
 * @public
 */
export declare class FASTHorizontalScroll extends FASTElement {
    /**
     * Reference to DOM element that scrolls the content
     * @public
     */
    scrollContainer: HTMLDivElement;
    /**
     * Reference to DOM element that holds the slotted content
     * @public
     */
    content: HTMLDivElement;
    /**
     * Reference to flipper to scroll to previous content
     * @public
     */
    previousFlipperContainer: HTMLDivElement;
    /**
     * Reference to flipper to scroll to the next content
     * @public
     */
    nextFlipperContainer: HTMLDivElement;
    /**
     * @internal
     */
    private framesPerSecond;
    /**
     * The calculated duration for a frame.
     *
     * @internal
     */
    private get frameTime();
    /**
     * The timeout identifier for the scroll event throttling.
     *
     * @internal
     */
    private resizeTimeout?;
    /**
     * The timeout identifier for the scroll event throttling.
     *
     * @internal
     */
    private scrollTimeout?;
    /**
     * Flag indicating that the items are being updated
     *
     * @internal
     */
    private updatingItems;
    /**
     * Speed of scroll in pixels per second
     * @public
     */
    speed: number;
    /**
     * The CSS time value for the scroll transition duration. Overrides the `speed` attribute.
     *
     * @remarks
     * When `duration` is set, the `speed` attribute has no effect.
     *
     * @public
     */
    duration: string;
    /**
     * Attribute used for easing, defaults to ease-in-out
     * @public
     */
    easing: ScrollEasing | string;
    /**
     * Attribute to hide flippers from assistive technology
     * @public
     */
    flippersHiddenFromAT: boolean;
    /**
     * Scrolling state
     * @internal
     */
    private scrolling;
    /**
     * Firing scrollstart and scrollend events
     * @internal
     */
    scrollingChanged(prev: unknown, next: boolean): void;
    /**
     * Detects if the component has been resized
     * @internal
     */
    private resizeDetector;
    /**
     * Width of the parent container
     * @internal
     */
    private width;
    /**
     * Scroll stop positions between elements
     * @internal
     */
    private scrollStops;
    /**
     * The default slotted items placed in the scrolling container.
     *
     * @public
     */
    scrollItems: HTMLElement[];
    /**
     * In RTL mode
     * @internal
     */
    private get isRtl();
    /**
     * View: default | mobile
     * @public
     */
    view: HorizontalScrollView;
    connectedCallback(): void;
    disconnectedCallback(): void;
    /**
     * Updates scroll stops and flippers when scroll items change
     * @param previous - current scroll items
     * @param next - new updated scroll items
     * @public
     */
    scrollItemsChanged(previous: HTMLElement[], next: HTMLElement[]): void;
    /**
     * destroys the instance's resize observer
     * @internal
     */
    private disconnectResizeDetector;
    /**
     * initializes the instance's resize observer
     * @internal
     */
    private initializeResizeDetector;
    /**
     * Looks for slots and uses child nodes instead
     * @internal
     */
    private updateScrollStops;
    /**
     * Finds all of the scroll stops between elements
     * @internal
     */
    private setStops;
    /**
     * Checks to see if the stops are returning values
     *  otherwise it will try to reinitialize them
     *
     * @returns boolean indicating that current scrollStops are valid non-zero values
     * @internal
     */
    private validateStops;
    /**
     *
     */
    private fixScrollMisalign;
    /**
     * Sets the controls view if enabled
     * @internal
     */
    private setFlippers;
    /**
     * Function that can scroll an item into view.
     * @param item - An item index, a scroll item or a child of one of the scroll items
     * @param padding - Padding of the viewport where the active item shouldn't be
     * @param rightPadding - Optional right padding. Uses the padding if not defined
     *
     * @public
     */
    scrollInView(item: HTMLElement | number, padding?: number, rightPadding?: number): void;
    /**
     * Lets the user arrow left and right through the horizontal scroll
     * @param e - Keyboard event
     * @public
     */
    keyupHandler(e: Event & KeyboardEvent): void;
    /**
     * Scrolls items to the left
     * @public
     */
    scrollToPrevious(): void;
    /**
     * Scrolls items to the right
     * @public
     */
    scrollToNext(): void;
    /**
     * Handles scrolling with easing
     * @param position - starting position
     * @param newPosition - position to scroll to
     * @public
     */
    scrollToPosition(newPosition: number, position?: number): void;
    /**
     * Monitors resize event on the horizontal-scroll element
     * @public
     */
    resized(): void;
    /**
     * Monitors scrolled event on the content container
     * @public
     */
    scrolled(): void;
}

/**
 * A Listbox Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#listbox | ARIA listbox }.
 *
 * @slot - The default slot for the listbox options
 *
 * @public
 */
export declare abstract class FASTListbox extends FASTElement {
    /**
     * The internal unfiltered list of selectable options.
     *
     * @internal
     */
    protected _options: FASTListboxOption[];
    /**
     * The first selected option.
     *
     * @internal
     */
    get firstSelectedOption(): FASTListboxOption;
    /**
     * Returns true if there is one or more selectable option.
     *
     * @internal
     */
    protected get hasSelectableOptions(): boolean;
    /**
     * The number of options.
     *
     * @public
     */
    get length(): number;
    /**
     * The list of options.
     *
     * @public
     */
    get options(): FASTListboxOption[];
    set options(value: FASTListboxOption[]);
    /**
     * Flag for the typeahead timeout expiration.
     *
     * @deprecated use `Listbox.typeaheadExpired`
     * @internal
     */
    protected get typeAheadExpired(): boolean;
    protected set typeAheadExpired(value: boolean);
    /**
     * The disabled state of the listbox.
     *
     * @public
     * @remarks
     * HTML Attribute: `disabled`
     */
    disabled: boolean;
    /**
     * The index of the selected option.
     *
     * @public
     */
    selectedIndex: number;
    /**
     * A collection of the selected options.
     *
     * @public
     */
    selectedOptions: FASTListboxOption[];
    /**
     * A standard `click` event creates a `focus` event before firing, so a
     * `mousedown` event is used to skip that initial focus.
     *
     * @internal
     */
    protected shouldSkipFocus: boolean;
    /**
     * A static filter to include only selectable options.
     *
     * @param n - element to filter
     * @public
     */
    static slottedOptionFilter: (n: HTMLElement) => boolean;
    /**
     * The default slotted elements.
     *
     * @internal
     */
    slottedOptions: Element[];
    /**
     * Typeahead timeout in milliseconds.
     *
     * @internal
     */
    protected static readonly TYPE_AHEAD_TIMEOUT_MS = 1000;
    /**
     * The current typeahead buffer string.
     *
     * @internal
     */
    protected typeaheadBuffer: string;
    /**
     * Flag for the typeahead timeout expiration.
     *
     * @internal
     */
    protected typeaheadExpired: boolean;
    /**
     * The timeout ID for the typeahead handler.
     *
     * @internal
     */
    protected typeaheadTimeout: number;
    /**
     * Handle click events for listbox options.
     *
     * @internal
     */
    clickHandler(e: MouseEvent): boolean | void;
    /**
     * Ensures that the provided option is focused and scrolled into view.
     *
     * @param optionToFocus - The option to focus
     * @internal
     */
    protected focusAndScrollOptionIntoView(optionToFocus?: FASTListboxOption | null): void;
    /**
     * Handles `focusin` actions for the component. When the component receives focus,
     * the list of selected options is refreshed and the first selected option is scrolled
     * into view.
     *
     * @internal
     */
    focusinHandler(e: FocusEvent): void;
    /**
     * Returns the options which match the current typeahead buffer.
     *
     * @internal
     */
    protected getTypeaheadMatches(): FASTListboxOption[];
    /**
     * Determines the index of the next option which is selectable, if any.
     *
     * @param prev - the previous selected index
     * @param next - the next index to select
     *
     * @internal
     */
    protected getSelectableIndex(prev: number | undefined, next: number): number;
    /**
     * Handles external changes to child options.
     *
     * @param source - the source object
     * @param propertyName - the property
     *
     * @internal
     */
    handleChange(source: any, propertyName: string): void;
    /**
     * Moves focus to an option whose label matches characters typed by the user.
     * Consecutive keystrokes are batched into a buffer of search text used
     * to match against the set of options.  If `TYPE_AHEAD_TIMEOUT_MS` passes
     * between consecutive keystrokes, the search restarts.
     *
     * @param key - the key to be evaluated
     *
     * @internal
     */
    handleTypeAhead(key: string): void;
    /**
     * Handles `keydown` actions for listbox navigation and typeahead.
     *
     * @internal
     */
    keydownHandler(e: KeyboardEvent): boolean | void;
    /**
     * Prevents `focusin` events from firing before `click` events when the
     * element is unfocused.
     *
     * @internal
     */
    mousedownHandler(e: MouseEvent): boolean | void;
    /**
     * Switches between single-selection and multi-selection mode.
     *
     * @param prev - the previous value of the `multiple` attribute
     * @param next - the next value of the `multiple` attribute
     *
     * @internal
     */
    multipleChanged(prev: boolean | undefined, next: boolean): void;
    /**
     * Updates the list of selected options when the `selectedIndex` changes.
     *
     * @param prev - the previous selected index value
     * @param next - the current selected index value
     *
     * @internal
     */
    selectedIndexChanged(prev: number | undefined, next: number): void;
    /**
     * Updates the selectedness of each option when the list of selected options changes.
     *
     * @param prev - the previous list of selected options
     * @param next - the current list of selected options
     *
     * @internal
     */
    protected selectedOptionsChanged(prev: FASTListboxOption[] | undefined, next: FASTListboxOption[]): void;
    /**
     * Moves focus to the first selectable option.
     *
     * @public
     */
    selectFirstOption(): void;
    /**
     * Moves focus to the last selectable option.
     *
     * @internal
     */
    selectLastOption(): void;
    /**
     * Moves focus to the next selectable option.
     *
     * @internal
     */
    selectNextOption(): void;
    /**
     * Moves focus to the previous selectable option.
     *
     * @internal
     */
    selectPreviousOption(): void;
    /**
     * Updates the selected index to match the first selected option.
     *
     * @internal
     */
    protected setDefaultSelectedOption(): void;
    /**
     * Sets an option as selected and gives it focus.
     *
     * @public
     */
    protected setSelectedOptions(): void;
    /**
     * Updates the list of options and resets the selected option when the slotted option content changes.
     *
     * @param prev - the previous list of slotted options
     * @param next - the current list of slotted options
     *
     * @internal
     */
    slottedOptionsChanged(prev: Element[] | undefined, next: Element[]): void;
    /**
     * Updates the filtered list of options when the typeahead buffer changes.
     *
     * @param prev - the previous typeahead buffer value
     * @param next - the current typeahead buffer value
     *
     * @internal
     */
    typeaheadBufferChanged(prev: string, next: string): void;
}

/**
 * @internal
 */
export declare interface FASTListbox extends DelegatesARIAListbox {
}

/**
 * A Listbox Custom HTML Element.
 * Implements the {@link https://w3c.github.io/aria/#listbox | ARIA listbox }.
 *
 * @public
 */
export declare class FASTListboxElement extends FASTListbox {
    /**
     * The index of the most recently checked option.
     *
     * @internal
     * @remarks
     * Multiple-selection mode only.
     */
    protected activeIndex: number;
    /**
     * Returns the last checked option.
     *
     * @internal
     */
    get activeOption(): FASTListboxOption | null;
    /**
     * Returns the list of checked options.
     *
     * @internal
     */
    protected get checkedOptions(): FASTListboxOption[];
    /**
     * Returns the index of the first selected option.
     *
     * @internal
     */
    get firstSelectedOptionIndex(): number;
    /**
     * Indicates if the listbox is in multi-selection mode.
     *
     * @remarks
     * HTML Attribute: `multiple`
     *
     * @public
     */
    multiple: boolean;
    /**
     * The start index when checking a range of options.
     *
     * @internal
     */
    protected rangeStartIndex: number;
    /**
     * The maximum number of options to display.
     *
     * @remarks
     * HTML Attribute: `size`.
     *
     * @public
     */
    size: number;
    /**
     * Updates the `ariaActiveDescendant` property when the active index changes.
     *
     * @param prev - the previous active index
     * @param next - the next active index
     *
     * @internal
     */
    protected activeIndexChanged(prev: number | undefined, next: number): void;
    /**
     * Toggles the checked state for the currently active option.
     *
     * @remarks
     * Multiple-selection mode only.
     *
     * @internal
     */
    protected checkActiveIndex(): void;
    /**
     * Sets the active index to the first option and marks it as checked.
     *
     * @remarks
     * Multi-selection mode only.
     *
     * @param preserveChecked - mark all options unchecked before changing the active index
     *
     * @internal
     */
    protected checkFirstOption(preserveChecked?: boolean): void;
    /**
     * Decrements the active index and sets the matching option as checked.
     *
     * @remarks
     * Multi-selection mode only.
     *
     * @param preserveChecked - mark all options unchecked before changing the active index
     *
     * @internal
     */
    protected checkLastOption(preserveChecked?: boolean): void;
    /**
     * @override
     * @internal
     */
    connectedCallback(): void;
    /**
     * @override
     * @internal
     */
    disconnectedCallback(): void;
    /**
     * Increments the active index and marks the matching option as checked.
     *
     * @remarks
     * Multiple-selection mode only.
     *
     * @param preserveChecked - mark all options unchecked before changing the active index
     *
     * @internal
     */
    protected checkNextOption(preserveChecked?: boolean): void;
    /**
     * Decrements the active index and marks the matching option as checked.
     *
     * @remarks
     * Multiple-selection mode only.
     *
     * @param preserveChecked - mark all options unchecked before changing the active index
     *
     * @internal
     */
    protected checkPreviousOption(preserveChecked?: boolean): void;
    /**
     * Handles click events for listbox options.
     *
     * @param e - the event object
     *
     * @override
     * @internal
     */
    clickHandler(e: MouseEvent): boolean | void;
    /**
     * @override
     * @internal
     */
    protected focusAndScrollOptionIntoView(): void;
    /**
     * In multiple-selection mode:
     * If any options are selected, the first selected option is checked when
     * the listbox receives focus. If no options are selected, the first
     * selectable option is checked.
     *
     * @override
     * @internal
     */
    focusinHandler(e: FocusEvent): boolean | void;
    /**
     * Unchecks all options when the listbox loses focus.
     *
     * @internal
     */
    focusoutHandler(e: FocusEvent): void;
    /**
     * Handles keydown actions for listbox navigation and typeahead
     *
     * @override
     * @internal
     */
    keydownHandler(e: KeyboardEvent): boolean | void;
    /**
     * Prevents `focusin` events from firing before `click` events when the
     * element is unfocused.
     *
     * @override
     * @internal
     */
    mousedownHandler(e: MouseEvent): boolean | void;
    /**
     * Switches between single-selection and multi-selection mode.
     *
     * @internal
     */
    multipleChanged(prev: boolean | undefined, next: boolean): void;
    /**
     * Sets an option as selected and gives it focus.
     *
     * @override
     * @public
     */
    protected setSelectedOptions(): void;
    /**
     * Ensures the size is a positive integer when the property is updated.
     *
     * @param prev - the previous size value
     * @param next - the current size value
     *
     * @internal
     */
    protected sizeChanged(prev: number | unknown, next: number): void;
    /**
     * Toggles the selected state of the provided options. If any provided items
     * are in an unselected state, all items are set to selected. If every
     * provided item is selected, they are all unselected.
     *
     * @internal
     */
    toggleSelectedForAllCheckedOptions(): void;
    /**
     * @override
     * @internal
     */
    typeaheadBufferChanged(prev: string, next: string): void;
    /**
     * Unchecks all options.
     *
     * @remarks
     * Multiple-selection mode only.
     *
     * @param preserveChecked - reset the rangeStartIndex
     *
     * @internal
     */
    protected uncheckAllOptions(preserveChecked?: boolean): void;
}

/**
 * An Option Custom HTML Element.
 * Implements {@link https://www.w3.org/TR/wai-aria-1.1/#option | ARIA option }.
 *
 * @slot start - Content which can be provided before the listbox option content
 * @slot end - Content which can be provided after the listbox option content
 * @slot - The default slot for listbox option content
 * @csspart content - Wraps the listbox option content
 *
 * @public
 */
export declare class FASTListboxOption extends FASTElement {
    /**
     * @internal
     */
    private _value;
    /**
     * @internal
     */
    proxy: HTMLOptionElement;
    /**
     * The checked state is used when the parent listbox is in multiple selection mode.
     * To avoid accessibility conflicts, the checked state should not be present in
     * single selection mode.
     *
     * @public
     */
    checked?: boolean;
    /**
     * Updates the ariaChecked property when the checked property changes.
     *
     * @param prev - the previous checked value
     * @param next - the current checked value
     *
     * @public
     */
    protected checkedChanged(prev: boolean | unknown, next?: boolean): void;
    /**
     * The default slotted content.
     *
     * @public
     */
    content: Node[];
    /**
     * Updates the proxy's text content when the default slot changes.
     * @param prev - the previous content value
     * @param next - the current content value
     *
     * @internal
     */
    protected contentChanged(prev: undefined | Node[], next: Node[]): void;
    /**
     * The defaultSelected state of the option.
     * @public
     */
    defaultSelected: boolean;
    protected defaultSelectedChanged(): void;
    /**
     * Tracks whether the "selected" property has been changed.
     * @internal
     */
    private dirtySelected;
    /**
     * The disabled state of the option.
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
    protected disabledChanged(prev: boolean, next: boolean): void;
    /**
     * The selected attribute value. This sets the initial selected value.
     *
     * @public
     * @remarks
     * HTML Attribute: selected
     */
    selectedAttribute: boolean;
    protected selectedAttributeChanged(): void;
    /**
     * The checked state of the control.
     *
     * @public
     */
    selected: boolean;
    protected selectedChanged(): void;
    /**
     * Track whether the value has been changed from the initial value
     */
    dirtyValue: boolean;
    /**
     * The initial value of the option. This value sets the `value` property
     * only when the `value` property has not been explicitly set.
     *
     * @remarks
     * HTML Attribute: value
     */
    protected initialValue: string;
    initialValueChanged(previous: string, next: string): void;
    get label(): string;
    get text(): string;
    set value(next: string);
    get value(): string;
    get form(): HTMLFormElement | null;
    constructor(text?: string, value?: string, defaultSelected?: boolean, selected?: boolean);
}

/**
 * @internal
 * @privateRemarks
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 */
export declare interface FASTListboxOption extends StartEnd, DelegatesARIAListboxOption {
}

/**
 * A Menu Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#menu | ARIA menu }.
 *
 * @slot - The default slot for the menu items
 *
 * @public
 */
export declare class FASTMenu extends FASTElement {
    /**
     * @internal
     */
    items: HTMLElement[];
    protected itemsChanged(oldValue: HTMLElement[], newValue: HTMLElement[]): void;
    protected menuItems: Element[] | undefined;
    private expandedItem;
    /**
     * The index of the focusable element in the items array
     * defaults to -1
     */
    private focusIndex;
    private static focusableElementRoles;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * @internal
     */
    disconnectedCallback(): void;
    /**
     * @internal
     */
    readonly isNestedMenu: () => boolean;
    /**
     * Focuses the first item in the menu.
     *
     * @public
     */
    focus(): void;
    /**
     * Collapses any expanded menu items.
     *
     * @public
     */
    collapseExpandedItem(): void;
    /**
     * @internal
     */
    handleMenuKeyDown(e: KeyboardEvent): void | boolean;
    /**
     * if focus is moving out of the menu, reset to a stable initial state
     * @internal
     */
    handleFocusOut: (e: FocusEvent) => void;
    private handleItemFocus;
    private handleExpandedChanged;
    private removeItemListeners;
    protected setItems(): void;
    handleChange(source: any, propertyName: string): void;
    /**
     * handle change from child element
     */
    private changeHandler;
    /**
     * check if the item is a menu item
     */
    protected isMenuItemElement: (el: Element) => el is HTMLElement;
    /**
     * check if the item is focusable
     */
    private isFocusableElement;
    private setFocus;
}

/**
 * A Switch Custom HTML Element.
 * Implements {@link https://www.w3.org/TR/wai-aria-1.1/#menuitem | ARIA menuitem }, {@link https://www.w3.org/TR/wai-aria-1.1/#menuitemcheckbox | ARIA menuitemcheckbox}, or {@link https://www.w3.org/TR/wai-aria-1.1/#menuitemradio | ARIA menuitemradio }.
 *
 * @slot checked-indicator - The checked indicator
 * @slot radio-indicator - The radio indicator
 * @slot start - Content which can be provided before the menu item content
 * @slot end - Content which can be provided after the menu item content
 * @slot - The default slot for menu item content
 * @slot expand-collapse-indicator - The expand/collapse indicator
 * @slot submenu - Used to nest menu's within menu items
 * @csspart input-container - The element representing the visual checked or radio indicator
 * @csspart checkbox - The element wrapping the `menuitemcheckbox` indicator
 * @csspart radio - The element wrapping the `menuitemradio` indicator
 * @csspart content - The element wrapping the menu item content
 * @csspart expand-collapse-glyph-container - The element wrapping the expand collapse element
 * @csspart expand-collapse - The expand/collapse element
 * @csspart submenu-region - The container for the submenu, used for positioning
 * @fires expanded-change - Fires a custom 'expanded-change' event when the expanded state changes
 * @fires change - Fires a custom 'change' event when a non-submenu item with a role of `menuitemcheckbox`, `menuitemradio`, or `menuitem` is invoked
 *
 * @public
 */
export declare class FASTMenuItem extends FASTElement {
    /**
     * The disabled state of the element.
     *
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
    /**
     * The expanded state of the element.
     *
     * @public
     * @remarks
     * HTML Attribute: expanded
     */
    expanded: boolean;
    protected expandedChanged(prev: boolean | undefined, next: boolean): void;
    /**
     * The role of the element.
     *
     * @public
     * @remarks
     * HTML Attribute: role
     */
    role: MenuItemRole;
    /**
     * Cleanup function for the submenu positioner.
     *
     * @public
     */
    cleanup: () => void;
    /**
     * The checked value of the element.
     *
     * @public
     * @remarks
     * HTML Attribute: checked
     */
    checked: boolean;
    protected checkedChanged(oldValue: boolean, newValue: boolean): void;
    /**
     * The hidden attribute.
     *
     * @public
     * @remarks
     * HTML Attribute: hidden
     */
    hidden: boolean;
    /**
     * The submenu slotted content.
     *
     * @internal
     */
    slottedSubmenu: HTMLElement[];
    /**
     * @internal
     */
    get hasSubmenu(): boolean;
    /**
     * Sets the submenu and updates its position.
     *
     * @internal
     */
    protected slottedSubmenuChanged(prev: HTMLElement[] | undefined, next: HTMLElement[]): void;
    /**
     * The container for the submenu.
     *
     * @internal
     */
    submenuContainer: HTMLDivElement;
    /**
     * @internal
     */
    submenu: HTMLElement | undefined;
    private focusSubmenuOnLoad;
    /**
     * @internal
     */
    disconnectedCallback(): void;
    /**
     * @internal
     */
    handleMenuItemKeyDown: (e: KeyboardEvent) => boolean;
    /**
     * @internal
     */
    handleMenuItemClick: (e: MouseEvent) => boolean;
    /**
     * @internal
     */
    submenuLoaded: () => void;
    /**
     * @internal
     */
    handleMouseOver: (e: MouseEvent) => boolean;
    /**
     * @internal
     */
    handleMouseOut: (e: MouseEvent) => boolean;
    /**
     * @internal
     */
    private closeSubMenu;
    /**
     * @internal
     */
    private expandAndFocus;
    /**
     * @internal
     */
    private invoke;
    /**
     * Calculate and apply submenu positioning.
     *
     * @public
     */
    updateSubmenu(): void;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface FASTMenuItem extends StartEnd {
}

/**
 * A Number Field Custom HTML Element.
 * Based largely on the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input/number | <input type="number" /> element }.
 *
 * @slot start - Content which can be provided before the number field input
 * @slot end - Content which can be provided after the number field input
 * @slot - The default slot for the label
 * @slot step-up-glyph - The glyph for the step up control
 * @slot step-down-glyph - The glyph for the step down control
 * @csspart label - The label
 * @csspart root - The element wrapping the control, including start and end slots
 * @csspart control - The element representing the input
 * @csspart controls - The step up and step down controls
 * @csspart step-up - The step up control
 * @csspart step-down - The step down control
 * @fires input - Fires a custom 'input' event when the value has changed
 * @fires change - Fires a custom 'change' event when the value has changed
 *
 * @public
 */
export declare class FASTNumberField extends FormAssociatedNumberField {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    /**
     * Indicates that this element should get focus after the page finishes loading. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#htmlattrdefautofocus | autofocus HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: autofocus
     */
    autofocus: boolean;
    /**
     * When true, spin buttons will not be rendered
     * @public
     * @remarks
     * HTML Attribute: autofocus
     */
    hideStep: boolean;
    /**
     * Sets the placeholder value of the element, generally used to provide a hint to the user.
     * @public
     * @remarks
     * HTML Attribute: placeholder
     * Using this attribute does is not a valid substitute for a labeling element.
     */
    placeholder: string;
    /**
     * Allows associating a {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/datalist | datalist} to the element by {@link https://developer.mozilla.org/en-US/docs/Web/API/Element/id}.
     * @public
     * @remarks
     * HTML Attribute: list
     */
    list: string;
    /**
     * The maximum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: maxlength
     */
    maxlength: number;
    /**
     * The minimum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: minlength
     */
    minlength: number;
    /**
     * Sets the width of the element to a specified number of characters.
     * @public
     * @remarks
     * HTMLAttribute: size
     */
    size: number;
    /**
     * Amount to increment or decrement the value by
     * @public
     * @remarks
     * HTMLAttribute: step
     */
    step: number;
    /**
     * The maximum the value can be
     * @public
     * @remarks
     * HTMLAttribute: max
     */
    max: number;
    /**
     * Ensures that the max is greater than the min and that the value
     *  is less than the max
     * @param previous - the previous max value
     * @param next - updated max value
     *
     * @internal
     */
    maxChanged(previous: number, next: number): void;
    /**
     * The minimum the value can be
     * @public
     * @remarks
     * HTMLAttribute: min
     */
    min: number;
    /**
     * Ensures that the min is less than the max and that the value
     *  is greater than the min
     * @param previous - previous min value
     * @param next - updated min value
     *
     * @internal
     */
    minChanged(previous: number, next: number): void;
    /**
     * The default slotted items
     * @internal
     */
    defaultSlottedNodes: Node[];
    /**
     * A reference to the internal input element
     * @internal
     */
    control: HTMLInputElement;
    /**
     * Flag to indicate that the value change is from the user input
     * @internal
     */
    private isUserInput;
    /**
     * The value property, typed as a number.
     *
     * @public
     */
    get valueAsNumber(): number;
    set valueAsNumber(next: number);
    /**
     * Validates that the value is a number between the min and max
     * @param previous - previous stored value
     * @param next - value being updated
     * @param updateControl - should the text field be updated with value, defaults to true
     * @internal
     */
    valueChanged(previous: string, next: string): void;
    /** {@inheritDoc (FormAssociated:interface).validate} */
    validate(): void;
    /**
     * Sets the internal value to a valid number between the min and max properties
     * @param value - user input
     *
     * @internal
     */
    private getValidValue;
    /**
     * Increments the value using the step value
     *
     * @public
     */
    stepUp(): void;
    /**
     * Decrements the value using the step value
     *
     * @public
     */
    stepDown(): void;
    /**
     * Sets up the initial state of the number field
     * @internal
     */
    connectedCallback(): void;
    /**
     * Selects all the text in the number field
     *
     * @public
     */
    select(): void;
    /**
     * Handles the internal control's `input` event
     * @internal
     */
    handleTextInput(): void;
    /**
     * Change event handler for inner control.
     * @remarks
     * "Change" events are not `composable` so they will not
     * permeate the shadow DOM boundary. This fn effectively proxies
     * the change event, emitting a `change` event whenever the internal
     * control emits a `change` event
     * @internal
     */
    handleChange(): void;
    /**
     * Handles the internal control's `keydown` event
     * @internal
     */
    handleKeyDown(e: KeyboardEvent): boolean;
    /**
     * Handles populating the input field with a validated value when
     *  leaving the input field.
     * @internal
     */
    handleBlur(): void;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface FASTNumberField extends StartEnd, DelegatesARIATextbox {
}

/**
 * A Picker Custom HTML Element.  This is an early "alpha" version of the component.
 * Developers should expect the api to evolve, breaking changes are possible.
 *
 * @beta
 */
export declare class FASTPicker extends FormAssociatedPicker {
    /**
     * Currently selected items. Comma delineated string ie. "apples,oranges".
     *
     * @remarks
     * HTML Attribute: selection
     */
    selection: string;
    protected selectionChanged(): void;
    /**
     * Currently available options. Comma delineated string ie. "apples,oranges".
     *
     * @remarks
     * HTML Attribute: options
     */
    options: string;
    protected optionsChanged(): void;
    /**
     * Whether the component should remove an option from the list when it is in the selection
     *
     * @remarks
     * HTML Attribute: filter-selected
     */
    filterSelected: boolean;
    /**
     * Whether the component should remove options based on the current query
     *
     * @remarks
     * HTML Attribute: filter-query
     */
    filterQuery: boolean;
    /**
     * The maximum number of items that can be selected.
     *
     * @remarks
     * HTML Attribute: max-selected
     */
    maxSelected: number | undefined;
    /**
     * The text to present to assistive technolgies when no suggestions are available.
     *
     * @remarks
     * HTML Attribute: no-suggestions-text
     */
    noSuggestionsText: string;
    /**
     *  The text to present to assistive technolgies when suggestions are available.
     *
     * @remarks
     * HTML Attribute: suggestions-available-text
     */
    suggestionsAvailableText: string;
    /**
     * The text to present to assistive technologies when suggestions are loading.
     *
     * @remarks
     * HTML Attribute: loading-text
     */
    loadingText: string;
    /**
     * Applied to the aria-label attribute of the input element
     *
     * @remarks
     * HTML Attribute: label
     */
    label: string;
    /**
     * Applied to the aria-labelledby attribute of the input element
     *
     * @remarks
     * HTML Attribute: labelledby
     */
    labelledBy: string;
    /**
     * Applied to the placeholder attribute of the input element
     *
     * @remarks
     * HTML Attribute: placholder
     */
    placeholder: string;
    /**
     * Controls menu placement
     *
     * @remarks
     * HTML Attribute: menu-placement
     */
    menuPlacement: MenuPlacement;
    protected menuPlacementChanged(): void;
    /**
     * Whether to display a loading state if the menu is opened.
     *
     */
    showLoading: boolean;
    protected showLoadingChanged(): void;
    /**
     * Template used to generate selected items.
     * This is used in a repeat directive.
     *
     */
    listItemTemplate: ViewTemplate;
    protected listItemTemplateChanged(): void;
    /**
     * Default template to use for selected items (usually specified in the component template).
     * This is used in a repeat directive.
     *
     */
    defaultListItemTemplate?: ViewTemplate;
    protected defaultListItemTemplateChanged(): void;
    /**
     * The item template currently in use.
     *
     * @internal
     */
    activeListItemTemplate?: ViewTemplate;
    /**
     * Template to use for available options.
     * This is used in a repeat directive.
     *
     */
    menuOptionTemplate: ViewTemplate;
    protected menuOptionTemplateChanged(): void;
    /**
     * Default template to use for available options (usually specified in the template).
     * This is used in a repeat directive.
     *
     */
    defaultMenuOptionTemplate?: ViewTemplate;
    protected defaultMenuOptionTemplateChanged(): void;
    /**
     * The option template currently in use.
     *
     * @internal
     */
    activeMenuOptionTemplate?: ViewTemplate;
    /**
     *  Template to use for the contents of a selected list item
     *
     */
    listItemContentsTemplate: ViewTemplate;
    /**
     *  Template to use for the contents of menu options
     *
     */
    menuOptionContentsTemplate: ViewTemplate;
    /**
     *  Current list of options in array form
     *
     */
    optionsList: string[];
    private optionsListChanged;
    /**
     * The text value currently in the input field
     *
     */
    query: string;
    protected queryChanged(): void;
    /**
     *  Current list of filtered options in array form
     *
     * @internal
     */
    filteredOptionsList: string[];
    protected filteredOptionsListChanged(): void;
    /**
     *  Indicates if the flyout menu is open or not
     *
     * @internal
     */
    flyoutOpen: boolean;
    protected flyoutOpenChanged(): void;
    /**
     *  The id of the menu element
     *
     * @internal
     */
    menuId: string;
    /**
     *  The tag for the selected list element (ie. "fast-picker-list" vs. "fluent-picker-list")
     *
     * @internal
     */
    selectedListTag: string;
    /**
     * The tag for the menu element (ie. "fast-picker-menu" vs. "fluent-picker-menu")
     *
     * @internal
     */
    menuTag: string;
    /**
     *  Index of currently active menu option
     *
     * @internal
     */
    menuFocusIndex: number;
    /**
     *  Id of currently active menu option.
     *
     * @internal
     */
    menuFocusOptionId: string | undefined;
    /**
     *  Internal flag to indicate no options available display should be shown.
     *
     * @internal
     */
    showNoOptions: boolean;
    private showNoOptionsChanged;
    /**
     *  The anchored region config to apply.
     *
     * @internal
     */
    menuConfig: AnchoredRegionConfig;
    /**
     *  Reference to the placeholder element for the repeat directive
     *
     */
    itemsPlaceholderElement: Node;
    /**
     * reference to the input element
     *
     * @internal
     */
    inputElement: HTMLInputElement;
    /**
     * reference to the selected list element
     *
     * @internal
     */
    listElement: FASTPickerList;
    /**
     * reference to the menu element
     *
     * @internal
     */
    menuElement: FASTPickerMenu;
    /**
     * reference to the anchored region element
     *
     * @internal
     */
    region: FASTAnchoredRegion;
    /**
     *
     *
     * @internal
     */
    selectedItems: string[];
    private optionsPlaceholder;
    private inputElementView;
    private behaviorOrchestrator;
    /**
     * @internal
     */
    connectedCallback(): void;
    disconnectedCallback(): void;
    /**
     * Move focus to the input element
     * @public
     */
    focus(): void;
    /**
     * Initialize the component.  This is delayed a frame to ensure children are connected as well.
     */
    private initialize;
    /**
     * Toggles the menu flyout
     */
    private toggleFlyout;
    /**
     * Handle input event from input element
     */
    private handleTextInput;
    /**
     * Handle click event from input element
     */
    private handleInputClick;
    /**
     * Handle the menu options updated event from the child menu
     */
    private handleMenuOptionsUpdated;
    /**
     * Handle key down events.
     */
    handleKeyDown(e: KeyboardEvent): boolean;
    /**
     * Handle focus in events.
     */
    handleFocusIn(e: FocusEvent): boolean;
    /**
     * Handle focus out events.
     */
    handleFocusOut(e: FocusEvent): boolean;
    /**
     * The list of selected items has changed
     */
    handleSelectionChange(): void;
    /**
     * Anchored region is loaded, menu and options exist in the DOM.
     */
    handleRegionLoaded(e: Event): void;
    /**
     * Sets properties on the anchored region once it is instanciated.
     */
    private setRegionProps;
    /**
     * Checks if the maximum number of items has been chosen and updates the ui.
     */
    private checkMaxItems;
    /**
     * A list item has been invoked.
     */
    handleItemInvoke(e: Event): boolean;
    /**
     * A menu option has been invoked.
     */
    handleOptionInvoke(e: Event): boolean;
    /**
     * Increments the focused list item by the specified amount
     */
    private incrementFocusedItem;
    /**
     * Disables the menu. Note that the menu can be open, just doens't have any valid options on display.
     */
    private disableMenu;
    /**
     * Sets the currently focused menu option by index
     */
    private setFocusedOption;
    /**
     * Updates the template used for the list item repeat behavior
     */
    private updateListItemTemplate;
    /**
     * Updates the template used for the menu option repeat behavior
     */
    private updateOptionTemplate;
    /**
     * Updates the filtered options array
     */
    private updateFilteredOptions;
    /**
     * Updates the menu configuration
     */
    private updateMenuConfig;
    /**
     * matches menu placement values with the associated menu config
     */
    private configLookup;
}

/**
 * A List Picker Menu Custom HTML Element.
 *
 * @beta
 */
export declare class FASTPickerList extends FASTElement {
}

/**
 * A picker list item Custom HTML Element.
 *
 * @beta
 */
export declare class FASTPickerListItem extends FASTElement {
    /**
     * The underlying string value of the item
     *
     * @remarks
     * HTML Attribute: value
     */
    value: string;
    /**
     *  The template used to render the contents of the list item
     *
     */
    contentsTemplate: ViewTemplate;
    protected contentsTemplateChanged(): void;
    private customView;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * @internal
     */
    disconnectedCallback(): void;
    handleKeyDown(e: KeyboardEvent): boolean;
    handleClick(e: MouseEvent): boolean;
    private handleInvoke;
    private updateView;
    private disconnectView;
}

/**
 * A List Picker Menu Custom HTML Element.
 *
 * @beta
 */
export declare class FASTPickerMenu extends FASTElement {
    /**
     *  Elements in the default slot
     *
     * @internal
     */
    menuElements: HTMLElement[];
    menuElementsChanged(): void;
    /**
     *  Elements in the header slot
     *
     *
     * @internal
     */
    headerElements: HTMLElement[];
    headerElementsChanged(): void;
    /**
     *  Elements in the footer slot
     *
     *
     * @internal
     */
    footerElements: HTMLElement[];
    footerElementsChanged(): void;
    /**
     * Text to display to assistive technology when
     * suggestions are available
     *
     */
    suggestionsAvailableText: string;
    /**
     * Children that are list items
     *
     * @internal
     */
    optionElements: HTMLElement[];
    private updateOptions;
    private addSlottedListItems;
}

/**
 * A picker list item Custom HTML Element.
 *
 * @beta
 */
export declare class FASTPickerMenuOption extends FASTElement {
    /**
     * The underlying string value of the item
     *
     * @remarks
     * HTML Attribute: value
     */
    value: string;
    /**
     *  The template used to render the contents of the list item
     */
    contentsTemplate: ViewTemplate;
    protected contentsTemplateChanged(): void;
    private customView;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * @internal
     */
    disconnectedCallback(): void;
    handleClick(e: MouseEvent): boolean;
    private handleInvoked;
    private updateView;
    private disconnectView;
}

/**
 * An Progress HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#progressbar | ARIA progressbar }.
 *
 * @slot indeterminate - The slot for a custom indeterminate indicator
 * @csspart progress - Represents the progress element
 * @csspart determinate - The determinate indicator
 * @csspart indeterminate - The indeterminate indicator
 *
 * @public
 */
export declare class FASTProgress extends FASTBaseProgress {
}

/**
 * An circular Progress HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#progressbar | ARIA progressbar }.
 *
 * @slot indeterminate - The slot for a custom indeterminate indicator
 * @slot determinate - The slot for a custom determinate indicator
 * @csspart progress - Represents the progress element
 * @csspart determinate - The determinate indicator
 * @csspart background - The background
 *
 * @public
 */
export declare class FASTProgressRing extends FASTBaseProgress {
}

/**
 * A Radio Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#radio | ARIA radio }.
 *
 * @slot checked-indicator - The checked indicator
 * @slot - The default slot for the label
 * @csspart control - The element representing the visual radio control
 * @csspart label - The label
 * @fires change - Emits a custom change event when the checked state changes
 *
 * @public
 */
export declare class FASTRadio extends FormAssociatedRadio implements RadioControl {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /**
     * The name of the radio. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#htmlattrdefname | name attribute} for more info.
     */
    name: string;
    /**
     * The element's value to be included in form submission when checked.
     * Default to "on" to reach parity with input[type="radio"]
     *
     * @internal
     */
    initialValue: string;
    /**
     * @internal
     */
    defaultSlottedNodes: Node[];
    /**
     * @internal
     */
    defaultCheckedChanged(): void;
    constructor();
    /**
     * @internal
     */
    connectedCallback(): void;
    private isInsideRadioGroup;
    /**
     * Handles key presses on the radio.
     * @beta
     */
    keypressHandler(e: KeyboardEvent): boolean | void;
    /**
     * Handles clicks on the radio.
     * @beta
     */
    clickHandler(e: MouseEvent): boolean | void;
}

/**
 * An Radio Group Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#radiogroup | ARIA radiogroup }.
 *
 * @slot label - The slot for the label
 * @slot - The default slot for radio buttons
 * @csspart positioning-region - The positioning region for laying out the radios
 * @fires change - Fires a custom 'change' event when the value changes
 *
 * @public
 */
export declare class FASTRadioGroup extends FASTElement {
    /**
     * When true, the child radios will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /**
     * Disables the radio group and child radios.
     *
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
    protected disabledChanged(): void;
    /**
     * The name of the radio group. Setting this value will set the name value
     * for all child radio elements.
     *
     * @public
     * @remarks
     * HTML Attribute: name
     */
    name: string;
    protected nameChanged(): void;
    /**
     * The value of the checked radio
     *
     * @public
     * @remarks
     * HTML Attribute: value
     */
    value: string;
    protected valueChanged(): void;
    /**
     * The orientation of the group
     *
     * @public
     * @remarks
     * HTML Attribute: orientation
     */
    orientation: Orientation | "horizontal" | "vertical";
    childItems: HTMLElement[];
    /**
     * @internal
     */
    slottedRadioButtons: HTMLElement[];
    protected slottedRadioButtonsChanged(oldValue: unknown, newValue: HTMLElement[]): void;
    private selectedRadio;
    private focusedRadio;
    private direction;
    private get parentToolbar();
    private get isInsideToolbar();
    private get isInsideFoundationToolbar();
    /**
     * @internal
     */
    connectedCallback(): void;
    disconnectedCallback(): void;
    private setupRadioButtons;
    private radioChangeHandler;
    private moveToRadioByIndex;
    private moveRightOffGroup;
    private moveLeftOffGroup;
    /**
     * @internal
     */
    focusOutHandler: (e: FocusEvent) => boolean | void;
    /**
     * @internal
     */
    clickHandler: (e: MouseEvent) => void;
    private shouldMoveOffGroupToTheRight;
    private shouldMoveOffGroupToTheLeft;
    private checkFocusedRadio;
    private moveRight;
    private moveLeft;
    /**
     * keyboard handling per https://w3c.github.io/aria-practices/#for-radio-groups-not-contained-in-a-toolbar
     * navigation is different when there is an ancestor with role='toolbar'
     *
     * @internal
     */
    keydownHandler: (e: KeyboardEvent) => boolean | void;
}

/**
 * A Search Custom HTML Element.
 * Based largely on the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input/search | <input type="search" /> element }.
 *
 * @slot start - Content which can be provided before the search input
 * @slot end - Content which can be provided after the search clear button
 * @slot - The default slot for the label
 * @slot clear-button - The clear button
 * @slot clear-glyph - The clear glyph
 * @csspart label - The label
 * @csspart root - The element wrapping the control, including start and end slots
 * @csspart control - The element representing the input
 * @csspart clear-button - The button to clear the input
 *
 * @public
 */
export declare class FASTSearch extends FormAssociatedSearch {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /**
     * Indicates that this element should get focus after the page finishes loading. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#htmlattrdefautofocus | autofocus HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: autofocus
     */
    autofocus: boolean;
    protected autofocusChanged(): void;
    /**
     * Sets the placeholder value of the element, generally used to provide a hint to the user.
     * @public
     * @remarks
     * HTML Attribute: placeholder
     * Using this attribute does is not a valid substitute for a labeling element.
     */
    placeholder: string;
    protected placeholderChanged(): void;
    /**
     * Allows associating a {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/datalist | datalist} to the element by {@link https://developer.mozilla.org/en-US/docs/Web/API/Element/id}.
     * @public
     * @remarks
     * HTML Attribute: list
     */
    list: string;
    protected listChanged(): void;
    /**
     * The maximum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: maxlength
     */
    maxlength: number;
    protected maxlengthChanged(): void;
    /**
     * The minimum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: minlength
     */
    minlength: number;
    protected minlengthChanged(): void;
    /**
     * A regular expression that the value must match to pass validation.
     * @public
     * @remarks
     * HTMLAttribute: pattern
     */
    pattern: string;
    protected patternChanged(): void;
    /**
     * Sets the width of the element to a specified number of characters.
     * @public
     * @remarks
     * HTMLAttribute: size
     */
    size: number;
    protected sizeChanged(): void;
    /**
     * Controls whether or not to enable spell checking for the input field, or if the default spell checking configuration should be used.
     * @public
     * @remarks
     * HTMLAttribute: size
     */
    spellcheck: boolean;
    protected spellcheckChanged(): void;
    /**
     * @internal
     */
    defaultSlottedNodes: Node[];
    /**
     * A reference to the internal close button element
     * @internal
     */
    root: HTMLDivElement;
    /**
     * A reference to the internal input element
     * @internal
     */
    control: HTMLInputElement;
    /**
     * @internal
     */
    connectedCallback(): void;
    /** {@inheritDoc (FormAssociated:interface).validate} */
    validate(): void;
    /**
     * Handles the internal control's `input` event
     * @internal
     */
    handleTextInput(): void;
    /**
     * Handles the control's clear value event
     * @public
     */
    handleClearInput(): void;
    /**
     * Change event handler for inner control.
     * @remarks
     * "Change" events are not `composable` so they will not
     * permeate the shadow DOM boundary. This fn effectively proxies
     * the change event, emitting a `change` event whenever the internal
     * control emits a `change` event
     * @internal
     */
    handleChange(): void;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface FASTSearch extends StartEnd, DelegatesARIASearch {
}

/**
 * A Select Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#select | ARIA select }.
 *
 * @slot start - Content which can be provided before the button content
 * @slot end - Content which can be provided after the button content
 * @slot button-container - The element representing the select button
 * @slot selected-value - The selected value
 * @slot indicator - The visual indicator for the expand/collapse state of the button
 * @slot - The default slot for slotted options
 * @csspart control - The element representing the select invoking element
 * @csspart selected-value - The element wrapping the selected value
 * @csspart indicator - The element wrapping the visual indicator
 * @csspart listbox - The listbox element
 * @fires input - Fires a custom 'input' event when the value updates
 * @fires change - Fires a custom 'change' event when the value updates
 *
 * @public
 */
export declare class FASTSelect extends FormAssociatedSelect {
    /**
     * The open attribute.
     *
     * @public
     * @remarks
     * HTML Attribute: open
     */
    open: boolean;
    /**
     * Sets focus and synchronizes ARIA attributes when the open property changes.
     *
     * @param prev - the previous open value
     * @param next - the current open value
     *
     * @internal
     */
    protected openChanged(prev: boolean | undefined, next: boolean): void;
    /**
     * The selectedIndex when the open property is true.
     *
     * @internal
     */
    private indexWhenOpened;
    /**
     * The internal value property.
     *
     * @internal
     */
    private _value;
    /**
     * The component is collapsible when in single-selection mode with no size attribute.
     *
     * @internal
     */
    get collapsible(): boolean;
    /**
     * The ref to the internal `.control` element.
     *
     * @internal
     */
    control: HTMLElement;
    /**
     * The value property.
     *
     * @public
     */
    get value(): string;
    set value(next: string);
    /**
     * Sets the value and display value to match the first selected option.
     *
     * @param shouldEmit - if true, the input and change events will be emitted
     *
     * @internal
     */
    private updateValue;
    /**
     * Updates the proxy value when the selected index changes.
     *
     * @param prev - the previous selected index
     * @param next - the next selected index
     *
     * @internal
     */
    selectedIndexChanged(prev: number | undefined, next: number): void;
    /**
     * Reference to the internal listbox element.
     *
     * @internal
     */
    listbox: HTMLDivElement;
    /**
     * The unique id for the internal listbox element.
     *
     * @internal
     */
    listboxId: string;
    /**
     * Cleanup function for the listbox positioner.
     *
     * @public
     */
    cleanup: () => void;
    /**
     * Calculate and apply listbox positioning based on available viewport space.
     *
     * @public
     */
    setPositioning(): void;
    /**
     * The value displayed on the button.
     *
     * @public
     */
    get displayValue(): string;
    /**
     * Synchronize the `aria-disabled` property when the `disabled` property changes.
     *
     * @param prev - The previous disabled value
     * @param next - The next disabled value
     *
     * @internal
     */
    disabledChanged(prev: boolean, next: boolean): void;
    /**
     * Reset the element to its first selectable option when its parent form is reset.
     *
     * @internal
     */
    formResetCallback(): void;
    /**
     * Handle opening and closing the listbox when the select is clicked.
     *
     * @param e - the mouse event
     * @internal
     */
    clickHandler(e: MouseEvent): boolean | void;
    /**
     * Handles focus state when the element or its children lose focus.
     *
     * @param e - The focus event
     * @internal
     */
    focusoutHandler(e: FocusEvent): boolean | void;
    /**
     * Updates the value when an option's value changes.
     *
     * @param source - the source object
     * @param propertyName - the property to evaluate
     *
     * @internal
     * @override
     */
    handleChange(source: any, propertyName: string): void;
    /**
     * Synchronize the form-associated proxy and updates the value property of the element.
     *
     * @param prev - the previous collection of slotted option elements
     * @param next - the next collection of slotted option elements
     *
     * @internal
     */
    slottedOptionsChanged(prev: Element[] | undefined, next: Element[]): void;
    /**
     * Prevents focus when size is set and a scrollbar is clicked.
     *
     * @param e - the mouse event object
     *
     * @override
     * @internal
     */
    mousedownHandler(e: MouseEvent): boolean | void;
    /**
     * Sets the multiple property on the proxy element.
     *
     * @param prev - the previous multiple value
     * @param next - the current multiple value
     */
    multipleChanged(prev: boolean | undefined, next: boolean): void;
    /**
     * Updates the selectedness of each option when the list of selected options changes.
     *
     * @param prev - the previous list of selected options
     * @param next - the current list of selected options
     *
     * @override
     * @internal
     */
    protected selectedOptionsChanged(prev: FASTListboxOption[] | undefined, next: FASTListboxOption[]): void;
    /**
     * Sets the selected index to match the first option with the selected attribute, or
     * the first selectable option.
     *
     * @override
     * @internal
     */
    protected setDefaultSelectedOption(): void;
    /**
     * Resets and fills the proxy to match the component's options.
     *
     * @internal
     */
    private setProxyOptions;
    /**
     * Handle keyboard interaction for the select.
     *
     * @param e - the keyboard event
     * @internal
     */
    keydownHandler(e: KeyboardEvent): boolean | void;
    connectedCallback(): void;
    disconnectedCallback(): void;
    /**
     * Updates the proxy's size property when the size attribute changes.
     *
     * @param prev - the previous size
     * @param next - the current size
     *
     * @override
     * @internal
     */
    protected sizeChanged(prev: number | undefined, next: number): void;
    /**
     *
     * @internal
     */
    private updateDisplayValue;
}

/**
 * @internal
 */
export declare interface FASTSelect extends StartEnd, DelegatesARIASelect {
}

/**
 * A Skeleton Custom HTML Element.
 *
 * @slot - The default slot
 *
 * @public
 */
export declare class FASTSkeleton extends FASTElement {
    /**
     * Indicates the Skeleton should have a filled style.
     *
     * @public
     * @remarks
     * HTML Attribute: fill
     */
    fill: string;
    /**
     * Indicates what the shape of the Skeleton should be.
     *
     * @public
     * @remarks
     * HTML Attribute: shape
     */
    shape: SkeletonShape;
    /**
     * Indicates that the component can accept a pattern URL.
     *
     * @public
     * @remarks
     * HTML Attribute: pattern
     */
    pattern: string;
    /**
     * Indicates that the component has an activated shimmer effect
     *
     * @public
     * @remarks
     * HTML Attribute: shimmer
     */
    shimmer: boolean;
}

/**
 * A Slider Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#slider | ARIA slider }.
 *
 * @slot track - The track of the slider
 * @slot track-start - The track-start visual indicator
 * @slot thumb - The slider thumb
 * @slot - The default slot for labels
 * @csspart positioning-region - The region used to position the elements of the slider
 * @csspart track-container - The region containing the track elements
 * @csspart track-start - The element wrapping the track start slot
 * @csspart thumb-container - The thumb container element which is programatically positioned
 * @fires change - Fires a custom 'change' event when the slider value changes
 *
 * @public
 */
export declare class FASTSlider extends FormAssociatedSlider implements SliderConfiguration {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     *
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /**
     * @internal
     */
    track: HTMLDivElement;
    /**
     * @internal
     */
    thumb: HTMLDivElement;
    /**
     * @internal
     */
    stepMultiplier: number;
    /**
     * @internal
     */
    direction: Direction;
    /**
     * @internal
     */
    isDragging: boolean;
    /**
     * @internal
     */
    position: string;
    /**
     * @internal
     */
    trackWidth: number;
    /**
     * @internal
     */
    trackMinWidth: number;
    /**
     * @internal
     */
    trackHeight: number;
    /**
     * @internal
     */
    trackLeft: number;
    /**
     * @internal
     */
    trackMinHeight: number;
    /**
     * The value property, typed as a number.
     *
     * @public
     */
    get valueAsNumber(): number;
    set valueAsNumber(next: number);
    /**
     * Custom function that generates a string for the component's "aria-valuetext" attribute based on the current value.
     *
     * @public
     */
    valueTextFormatter: (value: string) => string | null;
    /**
     * @internal
     */
    valueChanged(previous: string, next: string): void;
    /**
     * The minimum allowed value.
     *
     * @defaultValue - 0
     * @public
     * @remarks
     * HTML Attribute: min
     */
    min: number;
    private minChanged;
    /**
     * The maximum allowed value.
     *
     * @defaultValue - 10
     * @public
     * @remarks
     * HTML Attribute: max
     */
    max: number;
    private maxChanged;
    /**
     * Value to increment or decrement via arrow keys, mouse click or drag.
     *
     * @public
     * @remarks
     * HTML Attribute: step
     */
    step: number;
    private stepChanged;
    /**
     * The orientation of the slider.
     *
     * @public
     * @remarks
     * HTML Attribute: orientation
     */
    orientation: Orientation;
    private orientationChanged;
    /**
     * The selection mode.
     *
     * @public
     * @remarks
     * HTML Attribute: mode
     */
    mode: SliderMode;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * @internal
     */
    disconnectedCallback(): void;
    /**
     * Increment the value by the step
     *
     * @public
     */
    increment(): void;
    /**
     * Decrement the value by the step
     *
     * @public
     */
    decrement(): void;
    protected keypressHandler: (e: KeyboardEvent) => void;
    /**
     * Places the thumb based on the current value
     *
     * @public
     * @param direction - writing mode
     */
    private setThumbPositionForOrientation;
    /**
     * Update the step multiplier used to ensure rounding errors from steps that
     * are not whole numbers
     */
    private updateStepMultiplier;
    private setupTrackConstraints;
    private setupListeners;
    /**
     * @internal
     */
    initialValue: string;
    private get midpoint();
    private setupDefaultValue;
    /**
     *  Handle mouse moves during a thumb drag operation
     *  If the event handler is null it removes the events
     */
    private handleThumbMouseDown;
    /**
     *  Handle mouse moves during a thumb drag operation
     */
    private handleMouseMove;
    /**
     * Calculate the new value based on the given raw pixel value.
     *
     * @param rawValue - the value to be converted to a constrained value
     * @returns the constrained value
     *
     * @internal
     */
    calculateNewValue(rawValue: number): number;
    /**
     * Handle a window mouse up during a drag operation
     */
    private handleWindowMouseUp;
    private stopDragging;
    /**
     *
     * @param e - MouseEvent or null. If there is no event handler it will remove the events
     */
    private handleMouseDown;
    private convertToConstrainedValue;
}

/**
 * A label element intended to be used with the {@link @microsoft/fast-foundation#(FASTSlider:class)} component.
 *
 * @slot - The default slot for the label content
 * @csspart root - The element wrapping the label mark and text
 *
 * @public
 */
export declare class FASTSliderLabel extends FASTElement {
    /**
     * @internal
     */
    positionStyle: string;
    /**
     * @internal
     */
    root: HTMLDivElement;
    /**
     * The position of the label relative to the min and max value of the parent {@link @microsoft/fast-foundation#(FASTSlider:class)}.
     *
     * @public
     * @remarks
     * HTML Attribute: position
     */
    position: string;
    protected positionChanged(): void;
    /**
     * Hides the tick mark.
     *
     * @public
     * @remarks
     * HTML Attribute: hide-mark
     */
    hideMark: boolean;
    /**
     * The disabled state of the label. This is generally controlled by the parent {@link @microsoft/fast-foundation#(FASTSlider:class)}.
     *
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
    /**
     * The orientation state of the label. This is generally controlled by the parent {@link @microsoft/fast-foundation#(FASTSlider:class)}.
     *
     * @public
     * @deprecated - will be removed in coming ALPHA version
     * @remarks
     * HTML Attribute: orientation
     */
    orientation: Orientation;
    /**
     * @internal
     */
    protected orientationChanged(): void;
    /**
     * @internal
     */
    sliderMinPosition: number;
    /**
     * @internal
     */
    sliderMaxPosition: number;
    /**
     * @internal
     */
    sliderDirection: Direction;
    private notifier;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * @internal
     */
    disconnectedCallback(): void;
    /**
     * @internal
     */
    handleChange(source: any, propertyName: string): void;
    private isSliderConfig;
    private getSliderConfiguration;
    private positionAsStyle;
}

/**
 * A Switch Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#switch | ARIA switch }.
 *
 * @slot - The deafult slot for the label
 * @slot checked-message - The message when in a checked state
 * @slot unchecked-message - The message when in an unchecked state
 * @csspart label - The label
 * @csspart switch - The element representing the switch, which wraps the indicator
 * @csspart status-message - The wrapper for the status messages
 * @csspart checked-message - The checked message
 * @csspart unchecked-message - The unchecked message
 * @fires change - Emits a custom change event when the checked state changes
 *
 * @public
 */
export declare class FASTSwitch extends FormAssociatedSwitch {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /**
     * The element's value to be included in form submission when checked.
     * Default to "on" to reach parity with input[type="checkbox"]
     *
     * @internal
     */
    initialValue: string;
    /**
     * @internal
     */
    defaultSlottedNodes: Node[];
    constructor();
    /**
     * @internal
     */
    keypressHandler: (e: KeyboardEvent) => void;
    /**
     * @internal
     */
    clickHandler: (e: MouseEvent) => void;
    /**
     * @internal
     */
    checkedChanged(prev: boolean | undefined, next: boolean): void;
}

/**
 * A Tab Component to be used with {@link @microsoft/fast-foundation#(FASTTabs:class)}
 *
 * @slot - The default slot for the tab content
 *
 * @public
 */
export declare class FASTTab extends FASTElement {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/disabled | disabled HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface FASTTab extends StartEnd {
}

/**
 * A TabPanel Component to be used with {@link @microsoft/fast-foundation#(FASTTabs:class)}
 *
 * @slot - The default slot for the tabpanel content
 *
 * @public
 */
export declare class FASTTabPanel extends FASTElement {
}

/**
 * A Tabs Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#tablist | ARIA tablist }.
 *
 * @slot start - Content which can be provided before the tablist element
 * @slot end - Content which can be provided after the tablist element
 * @slot tab - The slot for tabs
 * @slot tabpanel - The slot for tabpanels
 * @csspart tablist - The element wrapping for the tabs
 * @csspart activeIndicator - The visual indicator
 * @fires change - Fires a custom 'change' event when a tab is clicked or during keyboard navigation
 *
 * @public
 */
export declare class FASTTabs extends FASTElement {
    /**
     * The orientation
     * @public
     * @remarks
     * HTML Attribute: orientation
     */
    orientation: TabsOrientation;
    /**
     * @internal
     */
    orientationChanged(): void;
    /**
     * The id of the active tab
     *
     * @public
     * @remarks
     * HTML Attribute: activeid
     */
    activeid: string;
    /**
     * @internal
     */
    activeidChanged(oldValue: string, newValue: string): void;
    /**
     * @internal
     */
    tabs: HTMLElement[];
    /**
     * @internal
     */
    tabsChanged(): void;
    /**
     * @internal
     */
    tabpanels: HTMLElement[];
    /**
     * @internal
     */
    tabpanelsChanged(): void;
    /**
     * Whether or not to show the active indicator
     * @public
     * @remarks
     * HTML Attribute: activeindicator
     */
    hideActiveIndicator: boolean;
    /**
     * @internal
     */
    activeIndicatorRef: HTMLElement;
    /**
     * @internal
     */
    showActiveIndicator: boolean;
    /**
     * A reference to the active tab
     * @public
     */
    activetab: HTMLElement;
    private prevActiveTabIndex;
    private activeTabIndex;
    private ticking;
    private tabIds;
    private tabpanelIds;
    private change;
    private isDisabledElement;
    private isFocusableElement;
    private getActiveIndex;
    private setTabs;
    private setTabPanels;
    private getTabIds;
    private getTabPanelIds;
    private setComponent;
    private handleTabClick;
    private isHorizontal;
    private handleTabKeyDown;
    private handleActiveIndicatorPosition;
    private animateActiveIndicator;
    /**
     * The adjust method for FASTTabs
     * @public
     * @remarks
     * This method allows the active index to be adjusted by numerical increments
     */
    adjust(adjustment: number): void;
    private adjustForward;
    private adjustBackward;
    private moveToTabByIndex;
    private focusTab;
    /**
     * @internal
     */
    connectedCallback(): void;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface FASTTabs extends StartEnd {
}

/**
 * A Text Area Custom HTML Element.
 * Based largely on the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/textarea | <textarea> element }.
 *
 * @slot - The default slot for the label
 * @csspart label - The label
 * @csspart root - The element wrapping the control
 * @csspart control - The textarea element
 * @fires change - Emits a custom 'change' event when the textarea emits a change event
 *
 * @public
 */
export declare class FASTTextArea extends FormAssociatedTextArea {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /**
     * The resize mode of the element.
     * @public
     * @remarks
     * HTML Attribute: resize
     */
    resize: TextAreaResize;
    /**
     * A reference to the internal textarea element
     * @internal
     */
    control: HTMLTextAreaElement;
    /**
     * Indicates that this element should get focus after the page finishes loading.
     * @public
     * @remarks
     * HTML Attribute: autofocus
     */
    autofocus: boolean;
    protected autofocusChanged(): void;
    /**
     * The {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Global_attributes/id | id} of the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/form | form} the element is associated to
     * @public
     */
    formId: string;
    /**
     * Allows associating a {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/datalist | datalist} to the element by {@link https://developer.mozilla.org/en-US/docs/Web/API/Element/id}.
     * @public
     * @remarks
     * HTML Attribute: list
     */
    list: string;
    protected listChanged(): void;
    /**
     * The maximum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: maxlength
     */
    maxlength: number;
    protected maxlengthChanged(): void;
    /**
     * The minimum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: minlength
     */
    minlength: number;
    protected minlengthChanged(): void;
    /**
     * The name of the element.
     * @public
     * @remarks
     * HTML Attribute: name
     */
    name: string;
    /**
     * Sets the placeholder value of the element, generally used to provide a hint to the user.
     * @public
     * @remarks
     * HTML Attribute: placeholder
     * Using this attribute does is not a valid substitute for a labeling element.
     */
    placeholder: string;
    /**
     * Sizes the element horizontally by a number of character columns.
     *
     * @public
     * @remarks
     * HTML Attribute: cols
     */
    cols: number;
    /**
     * Sizes the element vertically by a number of character rows.
     *
     * @public
     * @remarks
     * HTML Attribute: rows
     */
    rows: number;
    /**
     * Sets if the element is eligible for spell checking
     * but the UA.
     * @public
     * @remarks
     * HTML Attribute: spellcheck
     */
    spellcheck: boolean;
    protected spellcheckChanged(): void;
    /**
     * @internal
     */
    defaultSlottedNodes: Node[];
    /**
     * Selects all the text in the text area
     *
     * @public
     */
    select(): void;
    /**
     * @internal
     */
    handleTextInput: () => void;
    /**
     * Change event handler for inner control.
     * @remarks
     * "Change" events are not `composable` so they will not
     * permeate the shadow DOM boundary. This fn effectively proxies
     * the change event, emitting a `change` event whenever the internal
     * control emits a `change` event
     * @internal
     */
    handleChange(): void;
    /** {@inheritDoc (FormAssociated:interface).validate} */
    validate(): void;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface FASTTextArea extends DelegatesARIATextbox {
}

/**
 * A Text Field Custom HTML Element.
 * Based largely on the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input/text | <input type="text" /> element }.
 *
 * @slot start - Content which can be provided before the number field input
 * @slot end - Content which can be provided after the number field input
 * @slot - The default slot for the label
 * @csspart label - The label
 * @csspart root - The element wrapping the control, including start and end slots
 * @csspart control - The text field element
 * @fires change - Fires a custom 'change' event when the value has changed
 *
 * @public
 */
export declare class FASTTextField extends FormAssociatedTextField {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /**
     * Indicates that this element should get focus after the page finishes loading. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#htmlattrdefautofocus | autofocus HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: autofocus
     */
    autofocus: boolean;
    protected autofocusChanged(): void;
    /**
     * Sets the placeholder value of the element, generally used to provide a hint to the user.
     * @public
     * @remarks
     * HTML Attribute: placeholder
     * Using this attribute does is not a valid substitute for a labeling element.
     */
    placeholder: string;
    protected placeholderChanged(): void;
    /**
     * Allows setting a type or mode of text.
     * @public
     * @remarks
     * HTML Attribute: type
     */
    type: TextFieldType;
    private typeChanged;
    /**
     * Allows associating a {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/datalist | datalist} to the element by {@link https://developer.mozilla.org/en-US/docs/Web/API/Element/id}.
     * @public
     * @remarks
     * HTML Attribute: list
     */
    list: string;
    protected listChanged(): void;
    /**
     * The maximum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: maxlength
     */
    maxlength: number;
    protected maxlengthChanged(): void;
    /**
     * The minimum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: minlength
     */
    minlength: number;
    protected minlengthChanged(): void;
    /**
     * A regular expression that the value must match to pass validation.
     * @public
     * @remarks
     * HTMLAttribute: pattern
     */
    pattern: string;
    protected patternChanged(): void;
    /**
     * Sets the width of the element to a specified number of characters.
     * @public
     * @remarks
     * HTMLAttribute: size
     */
    size: number;
    protected sizeChanged(): void;
    /**
     * Controls whether or not to enable spell checking for the input field, or if the default spell checking configuration should be used.
     * @public
     * @remarks
     * HTMLAttribute: size
     */
    spellcheck: boolean;
    protected spellcheckChanged(): void;
    /**
     * @internal
     */
    defaultSlottedNodes: Node[];
    /**
     * A reference to the internal input element
     * @internal
     */
    control: HTMLInputElement;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * Selects all the text in the text field
     *
     * @public
     */
    select(): void;
    /**
     * Handles the internal control's `input` event
     * @internal
     */
    handleTextInput(): void;
    /**
     * Change event handler for inner control.
     * @remarks
     * "Change" events are not `composable` so they will not
     * permeate the shadow DOM boundary. This fn effectively proxies
     * the change event, emitting a `change` event whenever the internal
     * control emits a `change` event
     * @internal
     */
    handleChange(): void;
    /** {@inheritDoc (FormAssociated:interface).validate} */
    validate(): void;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast/issues/3317
 * @internal
 */
export declare interface FASTTextField extends StartEnd, DelegatesARIATextbox {
}

/**
 * A Toolbar Custom HTML Element.
 * Implements the {@link https://w3c.github.io/aria-practices/#Toolbar|ARIA Toolbar}.
 *
 * @slot start - Content which can be provided before the slotted items
 * @slot end - Content which can be provided after the slotted items
 * @slot - The default slot for slotted items
 * @slot label - The toolbar label
 * @csspart positioning-region - The element containing the items, start and end slots
 *
 * @public
 */
export declare class FASTToolbar extends FASTElement {
    /**
     * The internal index of the currently focused element.
     *
     * @internal
     */
    private _activeIndex;
    /**
     * The index of the currently focused element, clamped between 0 and the last element.
     *
     * @internal
     */
    get activeIndex(): number;
    set activeIndex(value: number);
    /**
     * The text direction of the toolbar.
     *
     * @internal
     */
    direction: Direction;
    /**
     * The collection of focusable toolbar controls.
     *
     * @internal
     */
    private focusableElements;
    /**
     * The orientation of the toolbar.
     *
     * @public
     * @remarks
     * HTML Attribute: `orientation`
     */
    orientation: Orientation;
    /**
     * The elements in the default slot.
     *
     * @internal
     */
    slottedItems: HTMLElement[];
    protected slottedItemsChanged(): void;
    /**
     * The elements in the label slot.
     *
     * @internal
     */
    slottedLabel: HTMLElement[];
    /**
     * Set the activeIndex when a focusable element in the toolbar is clicked.
     *
     * @internal
     */
    clickHandler(e: MouseEvent): boolean | void;
    childItems: Element[];
    protected childItemsChanged(prev: undefined | Element[], next: Element[]): void;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * When the toolbar receives focus, set the currently active element as focused.
     *
     * @internal
     */
    focusinHandler(e: FocusEvent): boolean | void;
    /**
     * Determines a value that can be used to iterate a list with the arrow keys.
     *
     * @param this - An element with an orientation and direction
     * @param key - The event key value
     * @internal
     */
    private getDirectionalIncrementer;
    /**
     * Handle keyboard events for the toolbar.
     *
     * @internal
     */
    keydownHandler(e: KeyboardEvent): boolean | void;
    /**
     * get all the slotted elements
     * @internal
     */
    protected get allSlottedItems(): (HTMLElement | Node)[];
    /**
     * Prepare the slotted elements which can be focusable.
     *
     * @internal
     */
    protected reduceFocusableElements(): void;
    /**
     * Set the activeIndex and focus the corresponding control.
     *
     * @param activeIndex - The new index to set
     * @internal
     */
    private setFocusedElement;
    /**
     * Reduce a collection to only its focusable elements.
     *
     * @param elements - Collection of elements to reduce
     * @param element - The current element
     *
     * @internal
     */
    private static reduceFocusableItems;
    /**
     * @internal
     */
    private setFocusableElements;
}

/**
 * @internal
 */
export declare interface FASTToolbar extends StartEnd, DelegatesARIAToolbar {
}

/**
 * An Tooltip Custom HTML Element.
 *
 * @slot - The default slot for the tooltip content
 * @csspart tooltip - The tooltip element
 * @fires dismiss - Fires a custom 'dismiss' event when the tooltip is visible and escape key is pressed
 *
 * @public
 */
export declare class FASTTooltip extends FASTElement {
    /**
     * Whether the tooltip is visible or not.
     * If undefined tooltip is shown when anchor element is hovered
     *
     * @defaultValue - undefined
     * @public
     * HTML Attribute: visible
     */
    visible: boolean;
    protected visibleChanged(): void;
    /**
     * The id of the element the tooltip is anchored to
     *
     * @defaultValue - undefined
     * @public
     * HTML Attribute: anchor
     */
    anchor: string;
    protected anchorChanged(): void;
    /**
     * The delay in milliseconds before a tooltip is shown after a hover event
     *
     * @defaultValue - 300
     * @public
     * HTML Attribute: delay
     */
    delay: number;
    /**
     * Controls the placement of the tooltip relative to the anchor.
     * When the position is undefined the tooltip is placed above or below the anchor based on available space.
     *
     * @defaultValue - undefined
     * @public
     * HTML Attribute: position
     */
    position: TooltipPosition;
    private positionChanged;
    /**
     * Controls when the tooltip updates its position, default is 'anchor' which only updates when
     * the anchor is resized.  'auto' will update on scroll/resize events.
     * Corresponds to anchored-region auto-update-mode.
     * @public
     * @remarks
     * HTML Attribute: auto-update-mode
     */
    autoUpdateMode: AutoUpdateMode;
    /**
     * Controls if the tooltip will always remain fully in the viewport on the horizontal axis
     * @public
     * @remarks
     * HTML Attribute: horizontal-viewport-lock
     */
    horizontalViewportLock: boolean;
    /**
     * Controls if the tooltip will always remain fully in the viewport on the vertical axis
     * @public
     * @remarks
     * HTML Attribute: vertical-viewport-lock
     */
    verticalViewportLock: boolean;
    /**
     * the html element currently being used as anchor.
     * Setting this directly overrides the anchor attribute.
     *
     * @public
     */
    anchorElement: HTMLElement | null;
    protected anchorElementChanged(oldValue: HTMLElement | null): void;
    /**
     * The current viewport element instance
     *
     * @internal
     */
    viewportElement: HTMLElement | null;
    protected viewportElementChanged(): void;
    /**
     * @internal
     * @defaultValue "dynamic"
     */
    verticalPositioningMode: AxisPositioningMode;
    /**
     * @internal
     * @defaultValue "dynamic"
     */
    horizontalPositioningMode: AxisPositioningMode;
    /**
     * @internal
     */
    horizontalInset: string;
    /**
     * @internal
     */
    verticalInset: string;
    /**
     * @internal
     */
    horizontalScaling: AxisScalingMode;
    /**
     * @internal
     */
    verticalScaling: AxisScalingMode;
    /**
     * @internal
     */
    verticalDefaultPosition: string | undefined;
    /**
     * @internal
     */
    horizontalDefaultPosition: string | undefined;
    /**
     * @internal
     */
    tooltipVisible: boolean;
    /**
     * Track current direction to pass to the anchored region
     * updated when tooltip is shown
     *
     * @internal
     */
    currentDirection: Direction;
    /**
     * reference to the anchored region
     *
     * @internal
     */
    region: FASTAnchoredRegion;
    /**
     * The timer that tracks delay time before the tooltip is shown on hover
     */
    private showDelayTimer;
    /**
     * The timer that tracks delay time before the tooltip is hidden
     */
    private hideDelayTimer;
    /**
     * Indicates whether the anchor is currently being hovered or has focus
     */
    private isAnchorHoveredFocused;
    /**
     * Indicates whether the region is currently being hovered
     */
    private isRegionHovered;
    connectedCallback(): void;
    disconnectedCallback(): void;
    /**
     * invoked when the anchored region's position relative to the anchor changes
     *
     * @internal
     */
    handlePositionChange: (ev: Event) => void;
    /**
     * mouse enters region
     */
    private handleRegionMouseOver;
    /**
     * mouse leaves region
     */
    private handleRegionMouseOut;
    /**
     * mouse enters anchor
     */
    private handleAnchorMouseOver;
    /**
     * mouse leaves anchor
     */
    private handleAnchorMouseOut;
    /**
     * anchor gets focus
     */
    private handleAnchorFocusIn;
    /**
     * anchor loses focus
     */
    private handleAnchorFocusOut;
    /**
     * starts the hide timer
     */
    private startHideDelayTimer;
    /**
     * clears the hide delay
     */
    private clearHideDelayTimer;
    /**
     * starts the show timer if not currently running
     */
    private startShowDelayTimer;
    /**
     * start hover
     */
    private startHover;
    /**
     * clears the show delay
     */
    private clearShowDelayTimer;
    /**
     * updated the properties being passed to the anchored region
     */
    private updateLayout;
    /**
     *  Gets the anchor element by id
     */
    private getAnchor;
    /**
     * handles key down events to check for dismiss
     */
    private handleDocumentKeydown;
    /**
     * determines whether to show or hide the tooltip based on current state
     */
    private updateTooltipVisibility;
    /**
     * shows the tooltip
     */
    private showTooltip;
    /**
     * hides the tooltip
     */
    private hideTooltip;
    /**
     * updates the tooltip anchored region props after it has been
     * added to the DOM
     */
    private setRegionProps;
}

/**
 * A Tree item Custom HTML Element.
 *
 * @slot start - Content which can be provided before the tree item content
 * @slot end - Content which can be provided after the tree item content
 * @slot - The default slot for tree item text content
 * @slot item - The slot for tree items (fast tree items manage this assignment themselves)
 * @slot expand-collapse-button - The expand/collapse button
 * @csspart positioning-region - The element used to position the tree item content with exception of any child nodes
 * @csspart content-region - The element containing the expand/collapse, start, and end slots
 * @csspart items - The element wrapping any child items
 * @csspart expand-collapse-button - The expand/collapse button
 * @fires expanded-change - Fires a custom 'expanded-change' event when the expanded state changes
 * @fires selected-change - Fires a custom 'selected-change' event when the selected state changes
 *
 * @public
 */
export declare class FASTTreeItem extends FASTElement {
    /**
     * When true, the control will be appear expanded by user interaction.
     * @public
     * @remarks
     * HTML Attribute: expanded
     */
    expanded: boolean;
    protected expandedChanged(prev: boolean | undefined, next: boolean): void;
    /**
     * When true, the control will appear selected by user interaction.
     * @public
     * @remarks
     * HTML Attribute: selected
     */
    selected: boolean;
    protected selectedChanged(prev: boolean | undefined, next: boolean): void;
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/disabled | disabled HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
    /**
     *  Reference to the expand/collapse button
     *
     * @internal
     */
    expandCollapseButton: HTMLDivElement;
    /**
     *  Readonly property identifying the element as a tree item
     *
     * @internal
     */
    readonly isTreeItem: boolean;
    /**
     * Whether the item is focusable
     *
     * @internal
     */
    focusable: boolean;
    /**
     *
     *
     * @internal
     */
    childItems: HTMLElement[];
    /**
     * The slotted child tree items
     *
     * @internal
     */
    items: HTMLElement[];
    protected itemsChanged(oldValue: unknown, newValue: HTMLElement[]): void;
    /**
     * Indicates if the tree item is nested
     *
     * @public
     * @deprecated - will be removed in coming ALPHA version
     * HTML Attribute: nested
     */
    nested: boolean;
    /**
     * Places document focus on a tree item
     *
     * @public
     * @param el - the element to focus
     */
    static focusItem(el: HTMLElement): void;
    /**
     * Whether the tree is nested
     *
     * @public
     */
    readonly isNestedItem: () => boolean;
    /**
     * Handle expand button click
     *
     * @internal
     */
    handleExpandCollapseButtonClick: (e: MouseEvent) => void;
    /**
     * Handle focus events
     *
     * @internal
     */
    handleFocus: (e: FocusEvent) => void;
    /**
     * Handle blur events
     *
     * @internal
     */
    handleBlur: (e: FocusEvent) => void;
    /**
     * Gets number of children
     *
     * @internal
     */
    get childItemLength(): number;
}

/**
 * Mark internal because exporting class and interface of the same name
 * confuses API documenter.
 * TODO: https://github.com/microsoft/fast-dna/issues/3317
 * @internal
 */
export declare interface FASTTreeItem extends StartEnd {
}

/**
 * A Tree view Custom HTML Element.
 * Implements the {@link https://w3c.github.io/aria-practices/#TreeView | ARIA TreeView }.
 *
 * @slot - The default slot for tree items
 *
 * @public
 */
export declare class FASTTreeView extends FASTElement {
    /**
     * The currently selected tree item
     * @public
     */
    currentSelected: HTMLElement | FASTTreeItem | null;
    /**
     *  Slotted children
     *
     * @internal
     */
    slottedTreeItems: HTMLElement[];
    protected slottedTreeItemsChanged(): void;
    /**
     * The tree item that is designated to be in the tab queue.
     *
     * @internal
     */
    currentFocused: HTMLElement | FASTTreeItem | null;
    /**
     * Handle focus events
     *
     * @internal
     */
    handleFocus: (e: FocusEvent) => void;
    /**
     * Handle blur events
     *
     * @internal
     */
    handleBlur: (e: FocusEvent) => void;
    /**
     * ref to the tree item
     *
     * @internal
     */
    treeView: HTMLElement;
    private nested;
    connectedCallback(): void;
    /**
     * KeyDown handler
     *
     *  @internal
     */
    handleKeyDown: (e: KeyboardEvent) => boolean | void;
    /**
     * Handles click events bubbling up
     *
     *  @internal
     */
    handleClick(e: Event): boolean | void;
    /**
     * Handles the selected-changed events bubbling up
     * from child tree items
     *
     *  @internal
     */
    handleSelectedChange: (e: Event) => boolean | void;
    /**
     * Move focus to a tree item based on its offset from the provided item
     */
    private focusNextNode;
    /**
     * Updates the tree view when slottedTreeItems changes
     */
    private setItems;
    /**
     * checks if there are any nested tree items
     */
    private getValidFocusableItem;
    /**
     * checks if there are any nested tree items
     */
    private checkForNestedItems;
    /**
     * check if the item is focusable
     */
    private isFocusableElement;
    private isSelectedElement;
    private getVisibleNodes;
}

/**
 * The direction options for flipper.
 * @public
 */
export declare const FlipperDirection: {
    readonly next: "next";
    readonly previous: "previous";
};

/**
 * The types for the flipper direction options.
 * @public
 */
export declare type FlipperDirection = typeof FlipperDirection[keyof typeof FlipperDirection];

/**
 * Flipper configuration options
 * @public
 */
export declare type FlipperOptions = {
    next?: string | SyntheticViewTemplate;
    previous?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#FASTFlipper} component.
 * @public
 */
export declare function flipperTemplate<T extends FASTFlipper>(options?: FlipperOptions): ElementViewTemplate<T>;

/**
 * A region that always places itself below the anchor, has
 * a width to match the anchor, and is sized vertically by content
 *
 * @public
 */
export declare const FlyoutPosBottom: AnchoredRegionConfig;

/**
 * A region that always places itself below the anchor, has
 * a width to match the anchor, and is sized vertically by available space
 *
 * @public
 */
export declare const FlyoutPosBottomFill: AnchoredRegionConfig;

/**
 * A region that places itself above or below the anchor
 * based on available space, has a width to match the anchor,
 * and is sized vertically by content
 *
 * @public
 */
export declare const FlyoutPosTallest: AnchoredRegionConfig;

/**
 * A region that places itself above or below the anchor
 * based on available space, has a width to match the anchor,
 * and is sized vertically by available space
 *
 * @public
 */
export declare const FlyoutPosTallestFill: AnchoredRegionConfig;

/**
 * A region that always places itself above the anchor, has
 * a width to match the anchor, and is sized vertically by content
 *
 * @public
 */
export declare const FlyoutPosTop: AnchoredRegionConfig;

/**
 * A region that always places itself above the anchor, has
 * a width to match the anchor, and is sized vertically by available space
 *
 * @public
 */
export declare const FlyoutPosTopFill: AnchoredRegionConfig;

/**
 * This will always evaluate to 'focus-visible' due to modern browser support.
 * @public
 * @deprecated - fast-foundation is removing styling utilities
 */
export declare const focusVisible = "focus-visible";

/**
 * This can be used to construct a behavior to apply a forced-colors only stylesheet.
 * @public
 */
export declare const forcedColorsStylesheetBehavior: (styles: ElementStyles) => MatchMediaStyleSheetBehavior;

/**
 * Base function for providing Custom Element Form Association.
 *
 * @beta
 */
export declare function FormAssociated<T extends ConstructableFormAssociated>(BaseCtor: T): T;

/**
 * Base class for providing Custom Element Form Association.
 *
 * @beta
 */
export declare interface FormAssociated extends Omit<ElementInternals_2, "labels"> {
    dirtyValue: boolean;
    disabled: boolean;
    readonly elementInternals: ElementInternals_2 | null;
    readonly formAssociated: boolean;
    initialValue: string;
    readonly labels: ReadonlyArray<Node[]>;
    name: string;
    required: boolean;
    value: string;
    currentValue: string;
    attachProxy(): void;
    detachProxy(): void;
    disabledChanged?(previous: boolean, next: boolean): void;
    formDisabledCallback?(disabled: boolean): void;
    formResetCallback(): void;
    initialValueChanged?(previous: string, next: string): void;
    nameChanged?(previous: string, next: string): void;
    requiredChanged(prev: boolean, next: boolean): void;
    stopPropagation(e: Event): void;
    /**
     * Sets the validity of the custom element. By default this uses the proxy element to determine
     * validity, but this can be extended or replaced in implementation.
     *
     * @param anchor - The anchor element to provide to ElementInternals.setValidity for surfacing the browser's constraint validation UI
     */
    validate(anchor?: HTMLElement): void;
    valueChanged(previous: string, next: string): void;
}

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(FASTButton:class)} component.
 *
 * @beta
 */
declare class FormAssociatedButton extends FormAssociatedButton_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedButton_base: typeof _Button;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(Checkbox:class)} component.
 *
 * @beta
 */
declare class FormAssociatedCheckbox extends FormAssociatedCheckbox_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedCheckbox_base: typeof _Checkbox;

/**
 * A form-associated base class for the {@link (Combobox:class)} component.
 *
 * @beta
 */
declare class FormAssociatedCombobox extends FormAssociatedCombobox_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedCombobox_base: typeof _Combobox;

/**
 * Combined type to describe a Form-associated element.
 *
 * @beta
 */
export declare type FormAssociatedElement = FormAssociated & FASTElement & HTMLElement & FormAssociatedProxy;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(NumberField:class)} component.
 *
 * @beta
 */
declare class FormAssociatedNumberField extends FormAssociatedNumberField_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedNumberField_base: typeof _NumberField;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(FASTPicker:class)} component.
 *
 * @beta
 */
declare class FormAssociatedPicker extends FormAssociatedPicker_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedPicker_base: typeof _Picker;

/**
 * Identifies a class as having a proxy element and optional submethods related
 * to the proxy element.
 *
 * @beta
 */
export declare interface FormAssociatedProxy {
    proxy: ProxyElement;
    disabledChanged?(previous: boolean, next: boolean): void;
    formDisabledCallback?(disabled: boolean): void;
    formResetCallback?(): void;
    initialValueChanged?(previous: string, next: string): void;
    valueChanged?(previous: string, next: string): void;
    nameChanged?(previous: string, next: string): void;
}

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(FASTRadio:class)} component.
 *
 * @beta
 */
declare class FormAssociatedRadio extends FormAssociatedRadio_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedRadio_base: typeof _Radio;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(FASTSearch:class)} component.
 *
 * @beta
 */
declare class FormAssociatedSearch extends FormAssociatedSearch_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedSearch_base: typeof _Search;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(Select:class)} component.
 *
 * @beta
 */
declare class FormAssociatedSelect extends FormAssociatedSelect_base {
    proxy: HTMLSelectElement;
}

declare const FormAssociatedSelect_base: typeof _Select;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(Slider:class)} component.
 *
 * @beta
 */
declare class FormAssociatedSlider extends FormAssociatedSlider_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedSlider_base: typeof _Slider;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(FASTSwitch:class)} component.
 *
 * @beta
 */
declare class FormAssociatedSwitch extends FormAssociatedSwitch_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedSwitch_base: typeof _Switch;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(FASTTextArea:class)} component.
 *
 * @beta
 */
declare class FormAssociatedTextArea extends FormAssociatedTextArea_base {
    proxy: HTMLTextAreaElement;
}

declare const FormAssociatedTextArea_base: typeof _TextArea;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(TextField:class)} component.
 *
 * @beta
 */
declare class FormAssociatedTextField extends FormAssociatedTextField_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedTextField_base: typeof _TextField;

/**
 * Enumerates the data grid auto generated header options
 * default option generates a non-sticky header row
 *
 * @public
 */
export declare const GenerateHeaderOptions: {
    readonly none: "none";
    readonly default: "default";
    readonly sticky: "sticky";
};

/**
 * The types for the data grid auto generated header options
 *
 * @public
 */
export declare type GenerateHeaderOptions = typeof GenerateHeaderOptions[keyof typeof GenerateHeaderOptions];

/**
 * Determines the current localization direction of an element.
 *
 * @param rootNode - the HTMLElement to begin the query from, usually "this" when used in a component controller
 * @returns the localization direction of the element
 *
 * @public
 */
export declare const getDirection: (rootNode: HTMLElement) => Direction;

/**
 * A CSS fragment to set `display: none;` when the host is hidden using the [hidden] attribute.
 * @public
 * @deprecated - fast-foundation is removing styling utilities
 */
export declare const hidden = ":host([hidden]){display:none}";

/**
 * Values for the horizontal positioning options for an anchored region
 * @public
 */
export declare const HorizontalPosition: {
    readonly start: "start";
    readonly end: "end";
    readonly left: "left";
    readonly right: "right";
    readonly center: "center";
    readonly unset: "unset";
};

/**
 * Type for the horizontal positioning options for an anchored region
 *
 * @public
 */
export declare type HorizontalPosition = typeof HorizontalPosition[keyof typeof HorizontalPosition];

/**
 * Horizontal scroll configuration options
 * @public
 */
export declare type HorizontalScrollOptions = StartEndOptions & {
    nextFlipper?: SyntheticViewTemplate | string;
    previousFlipper?: SyntheticViewTemplate | string;
};

/**
 * @public
 */
export declare function horizontalScrollTemplate<T extends FASTHorizontalScroll>(options?: HorizontalScrollOptions): ElementViewTemplate<T>;

/**
 * View options for the {@link @microsoft/fast-foundation#(FASTHorizontalScroll:class)|FASTHorizontalScroll} component.
 * @public
 */
export declare const HorizontalScrollView: {
    readonly default: "default";
    readonly mobile: "mobile";
};

/**
 * View option types for the {@link @microsoft/fast-foundation#(FASTHorizontalScroll:class)|FASTHorizontalScroll} component.
 * @public
 */
export declare type HorizontalScrollView = typeof HorizontalScrollView[keyof typeof HorizontalScrollView];

/**
 * Interactive template using DataGrid
 * @param context - The templates context
 * @param todayString - string representation of todays date
 * @returns - interactive calendar template
 *
 * @internal
 */
export declare function interactiveCalendarGridTemplate<T extends FASTCalendar>(options: CalendarOptions, todayString: string): ViewTemplate<T>;

/**
 * Determines if the element is a {@link (FASTListboxOption:class)}
 *
 * @param element - the element to test.
 * @public
 */
export declare function isListboxOption(el: Element): el is FASTListboxOption;

/**
 * check if the item is a tree item
 * @public
 * @remarks
 * determines if element is an HTMLElement and if it has the role treeitem
 */
export declare function isTreeItemElement(el: Element): el is HTMLElement;

/**
 * This can be used to construct a behavior to apply a prefers color scheme: light only stylesheet.
 * @public
 */
export declare const lightModeStylesheetBehavior: (styles: ElementStyles) => MatchMediaStyleSheetBehavior;

/**
 * Listbox option configuration options
 * @public
 */
export declare type ListboxOptionOptions = StartEndOptions;

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTListboxOption:class)} component.
 * @public
 */
export declare function listboxOptionTemplate<T extends FASTListboxOption>(options?: ListboxOptionOptions): ElementViewTemplate<T>;

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTListbox:class)} component.
 * @public
 */
export declare function listboxTemplate<T extends FASTListboxElement>(): ElementViewTemplate<T>;

/**
 * An abstract behavior to react to media queries. Implementations should implement
 * the `constructListener` method to perform some action based on media query changes.
 *
 * @public
 */
export declare abstract class MatchMediaBehavior implements HostBehavior {
    /**
     * The behavior needs to operate on element instances but elements might share a behavior instance.
     * To ensure proper attachment / detachment per instance, we construct a listener for
     * each bind invocation and cache the listeners by element reference.
     */
    private listenerCache;
    /**
     * The media query that the behavior operates on.
     */
    readonly query: MediaQueryList;
    /**
     *
     * @param query - The media query to operate from.
     */
    constructor(query: MediaQueryList);
    /**
     * Constructs a function that will be invoked with the MediaQueryList context
     * @param controller - The host controller orchestrating this behavior.
     */
    protected abstract constructListener(controller: HostController): MediaQueryListListener;
    /**
     * Binds the behavior to the element.
     * @param controller - The host controller orchestrating this behavior.
     */
    connectedCallback(controller: HostController): void;
    /**
     * Unbinds the behavior from the element.
     * @param controller - The host controller orchestrating this behavior.
     */
    disconnectedCallback(controller: HostController): void;
}

/**
 * A behavior to add or remove a stylesheet from an element based on a media query. The behavior ensures that
 * styles are applied while the a query matches the environment and that styles are not applied if the query does
 * not match the environment.
 *
 * @public
 */
export declare class MatchMediaStyleSheetBehavior extends MatchMediaBehavior {
    /**
     * The media query that the behavior operates on.
     */
    readonly query: MediaQueryList;
    /**
     * The styles object to be managed by the behavior.
     */
    readonly styles: ElementStyles;
    /**
     * Constructs a {@link MatchMediaStyleSheetBehavior} instance.
     * @param query - The media query to operate from.
     * @param styles - The styles to coordinate with the query.
     */
    constructor(query: MediaQueryList, styles: ElementStyles);
    /**
     * Defines a function to construct {@link MatchMediaStyleSheetBehavior | MatchMediaStyleSheetBehaviors} for
     * a provided query.
     * @param query - The media query to operate from.
     *
     * @public
     * @example
     *
     * ```ts
     * import { css } from "@microsoft/fast-element";
     * import { MatchMediaStyleSheetBehavior } from "@microsoft/fast-foundation";
     *
     * const landscapeBehavior = MatchMediaStyleSheetBehavior.with(
     *   window.matchMedia("(orientation: landscape)")
     * );
     *
     * const styles = css`
     *   :host {
     *     width: 200px;
     *     height: 400px;
     *   }
     * `
     * .withBehaviors(landscapeBehavior(css`
     *   :host {
     *     width: 400px;
     *     height: 200px;
     *   }
     * `))
     * ```
     */
    static with(query: MediaQueryList): (styles: ElementStyles) => MatchMediaStyleSheetBehavior;
    /**
     * Constructs a match-media listener for a provided element.
     * @param source - the element for which to attach or detach styles.
     */
    protected constructListener(controller: HostController): MediaQueryListListener;
    /**
     * Unbinds the behavior from the element.
     * @param controller - The host controller orchestrating this behavior.
     * @internal
     */
    removedCallback(controller: HostController<any>): void;
}

/**
 * An event listener fired by a {@link https://developer.mozilla.org/en-US/docs/Web/API/MediaQueryList | MediaQueryList }.
 * @public
 */
export declare type MediaQueryListListener = (this: MediaQueryList, ev?: MediaQueryListEvent) => void;

/**
 * Menu Item configuration options
 * @public
 */
export declare type MenuItemOptions = StartEndOptions & {
    checkboxIndicator?: string | SyntheticViewTemplate;
    expandCollapseGlyph?: string | SyntheticViewTemplate;
    radioIndicator?: string | SyntheticViewTemplate;
};

/**
 * Menu items roles.
 * @public
 */
export declare const MenuItemRole: {
    /**
     * The menu item has a "menuitem" role
     */
    readonly menuitem: "menuitem";
    /**
     * The menu item has a "menuitemcheckbox" role
     */
    readonly menuitemcheckbox: "menuitemcheckbox";
    /**
     * The menu item has a "menuitemradio" role
     */
    readonly menuitemradio: "menuitemradio";
};

/**
 * The types for menu item roles
 * @public
 */
export declare type MenuItemRole = typeof MenuItemRole[keyof typeof MenuItemRole];

/**
 * Generates a template for the {@link @microsoft/fast-foundation#(FASTMenuItem:class)} component using
 * the provided prefix.
 *
 * @public
 */
export declare function menuItemTemplate<T extends FASTMenuItem>(options?: MenuItemOptions): ElementViewTemplate<T>;

/**
 * vertical positioning values for an anchored region
 * @beta
 */
export declare const MenuPlacement: {
    readonly bottom: "bottom";
    readonly bottomFill: "bottom-fill";
    readonly tallest: "tallest";
    readonly tallestFill: "tallest-fill";
    readonly top: "top";
    readonly topFill: "top-fill";
};

/**
 * Type for vertical positioning values for an anchored region
 * @beta
 */
export declare type MenuPlacement = typeof MenuPlacement[keyof typeof MenuPlacement];

/**
 * The template for the {@link @microsoft/fast-foundation#FASTMenu} component.
 * @public
 */
export declare function menuTemplate<T extends FASTMenu>(): ElementViewTemplate<T>;

/**
 * Month format values for DateFormatter.
 * @public
 */
export declare const MonthFormat: {
    readonly "2-digit": "2-digit";
    readonly numeric: "numeric";
    readonly short: "short";
    readonly long: "long";
    readonly narrow: "narrow";
};

/**
 * Month format type for DateFormatter.
 * @public
 */
export declare type MonthFormat = typeof MonthFormat[keyof typeof MonthFormat];

/**
 * Information about a month
 * @public
 */
export declare type MonthInfo = {
    month: number;
    year: number;
    length: number;
    start: number;
};

/**
 * Non-interactive calendar template used for a readonly calendar
 * @param todayString - string representation of todays date
 * @returns - non-interactive calendar template
 *
 * @internal
 */
export declare function noninteractiveCalendarTemplate<T extends FASTCalendar>(options: CalendarOptions, todayString: string): ViewTemplate<T>;

declare class _NumberField extends FASTElement {
}

declare interface _NumberField extends FormAssociated {
}

/**
 * Number Field configuration options
 * @public
 */
export declare type NumberFieldOptions = StartEndOptions & {
    stepDownGlyph?: string | SyntheticViewTemplate;
    stepUpGlyph?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTNumberField:class)} component.
 * @public
 */
export declare function numberFieldTemplate<T extends FASTNumberField>(options?: NumberFieldOptions): ElementViewTemplate<T>;

declare class _Picker extends FASTElement {
}

declare interface _Picker extends FormAssociated {
}

/**
 *
 * @public
 */
export declare function pickerListItemTemplate<T extends FASTPickerListItem>(): ElementViewTemplate<T>;

/**
 *
 * @public
 */
export declare function pickerListTemplate<T extends FASTPickerList>(): ElementViewTemplate<T>;

/**
 *
 * @public
 */
export declare function pickerMenuOptionTemplate<T extends FASTPickerMenuOption>(): ElementViewTemplate<T>;

/**
 * The template for the List Picker component.
 * @public
 */
export declare function pickerMenuTemplate<T extends FASTPickerMenu>(): ElementViewTemplate<T>;

/**
 * Picker configuration options
 * @public
 */
export declare type PickerOptions = {
    anchoredRegion: TemplateElementDependency;
    pickerMenu: TemplateElementDependency;
    pickerMenuOption: TemplateElementDependency;
    pickerList: TemplateElementDependency;
    pickerListItem: TemplateElementDependency;
    progressRing: TemplateElementDependency;
};

/**
 * The template for the List Picker component.
 * @public
 */
export declare function pickerTemplate<T extends FASTPicker>(options: PickerOptions): ElementViewTemplate<T>;

/**
 * Progress configuration options
 * @public
 */
export declare type ProgressOptions = {
    indeterminateIndicator1?: string | SyntheticViewTemplate;
    indeterminateIndicator2?: string | SyntheticViewTemplate;
};

/**
 * ProgressRing configuration options
 * @public
 */
export declare type ProgressRingOptions = {
    indeterminateIndicator?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#FASTProgressRing} component.
 * @public
 */
export declare function progressRingTemplate<T extends FASTProgressRing>(options?: ProgressRingOptions): ElementViewTemplate<T>;

/**
 * The template for the {@link @microsoft/fast-foundation#FASTProgress} component.
 * @public
 */
export declare function progressTemplate<T extends FASTProgress>(options?: ProgressOptions): ElementViewTemplate<T>;

/**
 * A behavior to add or remove a stylesheet from an element based on a property. The behavior ensures that
 * styles are applied while the property matches and that styles are not applied if the property does
 * not match.
 *
 * @public
 */
export declare class PropertyStyleSheetBehavior implements HostBehavior {
    private propertyName;
    private value;
    private styles;
    /**
     * Constructs a {@link PropertyStyleSheetBehavior} instance.
     * @param propertyName - The property name to operate from.
     * @param value - The property value to operate from.
     * @param styles - The styles to coordinate with the property.
     */
    constructor(propertyName: string, value: any, styles: ElementStyles);
    /**
     * Binds the behavior to the element.
     * @param elementInstance - The element for which the property is applied.
     */
    addedCallback(controller: HostController): void;
    /**
     * Unbinds the behavior from the element.
     * @param source - The element for which the behavior is unbinding.
     */
    removedCallback(controller: HostController): void;
    /**
     * Change event for the provided element.
     * @param source - the element for which to attach or detach styles.
     * @param key - the key to lookup to know if the element already has the styles
     * @internal
     */
    handleChange(source: FASTElement, key: string): void;
}

/**
 * A target that can have key/value pairs set and removed.
 * @public
 */
export declare interface PropertyTarget {
    setProperty(name: string, value: string): void;
    removeProperty(name: string): void;
}

/**
 * Avaiable types for the `proxy` property.
 * @beta
 */
export declare type ProxyElement = HTMLSelectElement | HTMLTextAreaElement | HTMLInputElement;

declare class _Radio extends FASTElement {
}

declare interface _Radio extends CheckableFormAssociated {
}

/**
 * A structure representing a {@link @microsoft/fast-foundation#(FASTRadio:class)} element
 * @public
 */
export declare type RadioControl = Pick<HTMLInputElement, "checked" | "disabled" | "readOnly" | "focus" | "setAttribute" | "getAttribute">;

/**
 * The template for the {@link @microsoft/fast-foundation#FASTRadioGroup} component.
 * @public
 */
export declare function radioGroupTemplate<T extends FASTRadioGroup>(): ElementViewTemplate<T>;

/**
 * Radio configuration options
 * @public
 */
export declare type RadioOptions = {
    checkedIndicator?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTRadio:class)} component.
 * @public
 */
export declare function radioTemplate<T extends FASTRadio>(options?: RadioOptions): ElementViewTemplate<T>;

/**
 * Reflects attributes from the host element to the target element of the directive.
 * @param attributes - The attributes to reflect
 *
 * @beta
 * @example
 * ```ts
 * const template = html`
 *     <button
 *         ${reflectAttributes("aria-label", "aria-describedby")}
 *     >
 *          hello world
 *     </button
 * `
 * ```
 */
export declare function reflectAttributes<TSource = any, TParent = any>(...attributes: string[]): CaptureType<TSource, TParent>;

/**
 * @internal
 */
export declare const roleForMenuItem: {
    [value in keyof typeof MenuItemRole]: typeof MenuItemRole[value];
};

/**
 * Easing values for the {@link @microsoft/fast-foundation#(FASTHorizontalScroll:class)|FASTHorizontalScroll} component.
 * @public
 */
export declare const ScrollEasing: {
    readonly linear: "linear";
    readonly easeIn: "ease-in";
    readonly easeOut: "ease-out";
    readonly easeInOut: "ease-in-out";
};

/**
 * Easing types for the {@link @microsoft/fast-foundation#(FASTHorizontalScroll:class)|FASTHorizontalScroll} component.
 * @public
 */
export declare type ScrollEasing = typeof ScrollEasing[keyof typeof ScrollEasing];

declare class _Search extends FASTElement {
}

declare interface _Search extends FormAssociated {
}

/**
 * Search configuration options
 * @public
 */
export declare type SearchOptions = StartEndOptions;

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTSearch:class)} component.
 * @public
 */
export declare function searchTemplate<T extends FASTSearch>(options?: SearchOptions): ElementViewTemplate<T>;

declare class _Select extends FASTListboxElement {
}

declare interface _Select extends FormAssociated {
}

/**
 * Select configuration options
 * @public
 */
export declare type SelectOptions = StartEndOptions & {
    indicator?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTSelect:class)} component.
 * @public
 */
export declare function selectTemplate<T extends FASTSelect>(options?: SelectOptions): ElementViewTemplate<T>;

/**
 * A structure representing skeleton shapes
 * @public
 */
export declare const SkeletonShape: {
    readonly rect: "rect";
    readonly circle: "circle";
};

/**
 * @public
 */
export declare type SkeletonShape = typeof SkeletonShape[keyof typeof SkeletonShape];

/**
 * The template for the fast-skeleton component
 * @public
 */
export declare function skeletonTemplate<T extends FASTSkeleton>(): ElementViewTemplate<T>;

declare class _Slider extends FASTElement {
}

declare interface _Slider extends FormAssociated {
}

/**
 * The configuration structure of {@link @microsoft/fast-foundation#(FASTSlider:class)}.
 * @public
 */
export declare interface SliderConfiguration {
    max: number;
    min: number;
    orientation?: Orientation;
    direction?: Direction;
    disabled?: boolean;
}

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTSliderLabel:class)} component.
 * @public
 */
export declare function sliderLabelTemplate<T extends FASTSliderLabel>(): ElementViewTemplate<T>;

/**
 * The selection modes of a {@link @microsoft/fast-foundation#(FASTSlider:class)}.
 * @public
 */
export declare const SliderMode: {
    readonly singleValue: "single-value";
};

/**
 * The types for the selection mode of the slider
 * @public
 */
export declare type SliderMode = typeof SliderMode[keyof typeof SliderMode];

/**
 * Slider configuration options
 * @public
 */
export declare type SliderOptions = {
    thumb?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTSlider:class)} component.
 * @public
 */
export declare function sliderTemplate<T extends FASTSlider>(options?: SliderOptions): ElementViewTemplate<T>;

/**
 * A mixin class implementing start and end slots.
 * These are generally used to decorate text elements with icons or other visual indicators.
 * @public
 */
export declare class StartEnd {
    start: HTMLSlotElement;
    end: HTMLSlotElement;
}

/**
 * Start/End configuration options
 * @public
 */
export declare type StartEndOptions = StartOptions & EndOptions;

/**
 * Start configuration options
 * @public
 */
export declare type StartOptions = {
    start?: string | SyntheticViewTemplate;
};

/**
 * The template for the start slots.
 * For use with {@link StartEnd}
 *
 * @public
 */
export declare function startSlotTemplate(options: StartOptions): ViewTemplate<StartEnd>;

/**
 * A design token value with no observable dependencies
 * @public
 */
export declare type StaticDesignTokenValue<T> = T extends (...args: any[]) => any ? DerivedDesignTokenValue<T> : T;

/**
 * @alpha
 */
export declare const supportsElementInternals: boolean;

declare class _Switch extends FASTElement {
}

declare interface _Switch extends CheckableFormAssociated {
}

/**
 * Switch configuration options
 * @public
 */
export declare type SwitchOptions = {
    switch?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTSwitch:class)} component.
 * @public
 */
export declare function switchTemplate<T extends FASTSwitch>(options?: SwitchOptions): ElementViewTemplate<T>;

/**
 * Tab option configuration options
 * @public
 */
export declare type TabOptionOptions = StartEndOptions;

/**
 * The template for the {@link @microsoft/fast-foundation#FASTTabPanel} component.
 * @public
 */
export declare function tabPanelTemplate<T extends FASTTabPanel>(): ElementViewTemplate<T>;

/**
 * Tabs option configuration options
 * @public
 */
export declare type TabsOptions = StartEndOptions;

/**
 * The orientation of the {@link @microsoft/fast-foundation#(FASTTabs:class)} component
 * @public
 */
export declare const TabsOrientation: {
    readonly vertical: "vertical";
    readonly horizontal: "horizontal";
};

/**
 * The types for the Tabs component
 * @public
 */
export declare type TabsOrientation = typeof TabsOrientation[keyof typeof TabsOrientation];

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTTabs:class)} component.
 * @public
 */
export declare function tabsTemplate<T extends FASTTabs>(options?: TabsOptions): ElementViewTemplate<T>;

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTTab:class)} component.
 * @public
 */
export declare function tabTemplate<T extends FASTTab>(options?: StartEndOptions): ElementViewTemplate<T>;

/**
 * Determines what HTML tag name to use for the dependency.
 * @param dependency - The dependency the template is dependent on.
 * @returns The tag name to use in markup.
 * @beta
 */
export declare function tagFor(dependency: TemplateElementDependency): string;

/**
 * Used to designate a template's dependency on another custom element.
 * @beta
 */
export declare type TemplateElementDependency = string | FASTElementDefinition | Constructable<FASTElement>;

declare class _TextArea extends FASTElement {
}

declare interface _TextArea extends FormAssociated {
}

/**
 * Resize mode for a TextArea
 * @public
 */
export declare const TextAreaResize: {
    /**
     * No resize.
     */
    readonly none: "none";
    /**
     * Resize vertically and horizontally.
     */
    readonly both: "both";
    /**
     * Resize horizontally.
     */
    readonly horizontal: "horizontal";
    /**
     * Resize vertically.
     */
    readonly vertical: "vertical";
};

/**
 * Types for the Text Area resize mode
 * @public
 */
export declare type TextAreaResize = typeof TextAreaResize[keyof typeof TextAreaResize];

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTTextArea:class)} component.
 * @public
 */
export declare function textAreaTemplate<T extends FASTTextArea>(): ElementViewTemplate<T>;

declare class _TextField extends FASTElement {
}

declare interface _TextField extends FormAssociated {
}

/**
 * Text field configuration options
 * @public
 */
export declare type TextFieldOptions = StartEndOptions;

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTTextField:class)} component.
 * @public
 */
export declare function textFieldTemplate<T extends FASTTextField>(options?: TextFieldOptions): ElementViewTemplate<T>;

/**
 * Text field sub-types
 * @public
 */
export declare const TextFieldType: {
    /**
     * An email TextField
     */
    readonly email: "email";
    /**
     * A password TextField
     */
    readonly password: "password";
    /**
     * A telephone TextField
     */
    readonly tel: "tel";
    /**
     * A text TextField
     */
    readonly text: "text";
    /**
     * A URL TextField
     */
    readonly url: "url";
};

/**
 * Types for the text field sub-types
 * @public
 */
export declare type TextFieldType = typeof TextFieldType[keyof typeof TextFieldType];

/**
 * Toolbar configuration options
 * @public
 */
export declare type ToolbarOptions = StartEndOptions;

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTToolbar:class)} component.
 *
 * @public
 */
export declare function toolbarTemplate<T extends FASTToolbar>(options?: ToolbarOptions): ElementViewTemplate<T>;

/**
 * Options for the tooltip.
 * @public
 */
export declare type TooltipOptions = {
    anchoredRegion: TemplateElementDependency;
};

/**
 * Enumerates possible tooltip positions
 *
 * @public
 */
export declare const TooltipPosition: {
    /**
     * The tooltip is positioned above the element
     */
    readonly top: "top";
    /**
     * The tooltip is positioned to the right of the element
     */
    readonly right: "right";
    /**
     * The tooltip is positioned below the element
     */
    readonly bottom: "bottom";
    /**
     * The tooltip is positioned to the left of the element
     */
    readonly left: "left";
    /**
     * The tooltip is positioned in the center of the element
     */
    readonly center: "center";
    /**
     * The tooltip is positioned before the element
     */
    readonly start: "start";
    /**
     * The tooltip is positioned after the element
     */
    readonly end: "end";
    /**
     * The tooltip is positioned above the element and to the left
     */
    readonly topLeft: "top-left";
    /**
     * The tooltip is positioned above the element and horizontally centered
     */
    readonly topCenter: "top-center";
    /**
     * The tooltip is positioned above the element and to the right
     */
    readonly topRight: "top-right";
    /**
     * The tooltip is positioned below the element and to the left
     */
    readonly bottomLeft: "bottom-left";
    /**
     * The tooltip is positioned below the element and horizontally centered
     */
    readonly bottomCenter: "bottom-center";
    /**
     * The tooltip is positioned below the element and to the right
     */
    readonly bottomRight: "bottom-right";
    /**
     * The tooltip is positioned above the element and to the left
     */
    readonly topStart: "top-start";
    /**
     * The tooltip is positioned above the element and to the right
     */
    readonly topEnd: "top-end";
    /**
     * The tooltip is positioned below the element and to the left
     */
    readonly bottomStart: "bottom-start";
    /**
     * The tooltip is positioned below the element and to the right
     */
    readonly bottomEnd: "bottom-end";
};

/**
 * The possible tooltip positions
 *
 * @public
 */
export declare type TooltipPosition = typeof TooltipPosition[keyof typeof TooltipPosition];

/**
 * Creates a template for the {@link @microsoft/fast-foundation#(FASTTooltip:class)} component using the provided prefix.
 * @public
 */
export declare function tooltipTemplate<T extends FASTTooltip>(options: TooltipOptions): ElementViewTemplate<T>;

/**
 * Tree Item configuration options
 * @public
 */
export declare type TreeItemOptions = StartEndOptions & {
    expandCollapseGlyph?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(FASTTreeItem:class)} component.
 * @public
 */
export declare function treeItemTemplate<T extends FASTTreeItem>(options?: TreeItemOptions): ElementViewTemplate<T>;

/**
 * The template for the {@link @microsoft/fast-foundation#FASTTreeView} component.
 * @public
 */
export declare function treeViewTemplate<T extends FASTTreeView>(): ElementViewTemplate<T>;

/**
 * This file enables typing support for ElementInternals APIs.
 * It is largely taken from https://github.com/microsoft/TSJS-lib-generator/pull/818/files.
 *
 * When TypeScript adds support for these APIs we can delete this file.
 */
declare interface ValidityStateFlags {
    badInput?: boolean;
    customError?: boolean;
    patternMismatch?: boolean;
    rangeOverflow?: boolean;
    rangeUnderflow?: boolean;
    stepMismatch?: boolean;
    tooLong?: boolean;
    tooShort?: boolean;
    typeMismatch?: boolean;
    valueMissing?: boolean;
}

/**
 * Values for the vertical positioning options for an anchored region
 * @public
 */
export declare const VerticalPosition: {
    readonly top: "top";
    readonly bottom: "bottom";
    readonly center: "center";
    readonly unset: "unset";
};

/**
 * Type for the vertical positioning options for an anchored region
 *
 * @public
 */
export declare type VerticalPosition = typeof VerticalPosition[keyof typeof VerticalPosition];

/**
 * Weekday format values for DateFormatter.
 * @public
 */
export declare const WeekdayFormat: {
    readonly long: "long";
    readonly narrow: "narrow";
    readonly short: "short";
};

/**
 * Weekday format type for DateFormatter.
 * @public
 */
export declare type WeekdayFormat = typeof WeekdayFormat[keyof typeof WeekdayFormat];

/**
 * Calendar weekday text.
 * @public
 */
export declare type WeekdayText = {
    text: string;
    abbr?: string;
};

/**
 * filters out any whitespace-only nodes, to be used inside a template.
 *
 * @param value - The Node that is being inspected
 * @param index - The index of the node within the array
 * @param array - The Node array that is being filtered
 * @returns true if the node is not a whitespace-only node, false otherwise
 *
 * @public
 */
export declare const whitespaceFilter: ElementsFilter;

/**
 * Year format values for DateFormatter.
 * @public
 */
export declare const YearFormat: {
    readonly "2-digit": "2-digit";
    readonly numeric: "numeric";
};

/**
 * Year format type for DateFormatter.
 * @public
 */
export declare type YearFormat = typeof YearFormat[keyof typeof YearFormat];

export { }
